@@ -44,8 +44,12 @@
 #include "qwaylandscreen_p.h"
 #include "qwaylandcursor_p.h"
 #include "qwaylandinputdevice_p.h"
+#if QT_CONFIG(clipboard)
 #include "qwaylandclipboard_p.h"
+#endif
+#if QT_CONFIG(wayland_datadevice)
 #include "qwaylanddatadevicemanager_p.h"
+#endif
 #include "qwaylandhardwareintegration_p.h"
 #include "qwaylandxdgshell_p.h"
 #include "qwaylandxdgsurface_p.h"
@@ -84,7 +88,8 @@ struct wl_surface *QWaylandDisplay::createSurface(void *handle)
 
 QWaylandShellSurface *QWaylandDisplay::createShellSurface(QWaylandWindow *window)
 {
-    Q_ASSERT(mWaylandIntegration->shellIntegration());
+    if (!mWaylandIntegration->shellIntegration())
+        return 0;
     return mWaylandIntegration->shellIntegration()->createShellSurface(window);
 }
 
@@ -119,7 +124,9 @@ QWaylandWindowManagerIntegration *QWaylandDisplay::windowManagerIntegration() co
 
 QWaylandDisplay::QWaylandDisplay(QWaylandIntegration *waylandIntegration)
     : mWaylandIntegration(waylandIntegration)
+#if QT_CONFIG(wayland_datadevice)
     , mDndSelectionHandler(0)
+#endif
     , mWindowExtension(0)
     , mSubCompositor(0)
     , mTouchExtension(0)
@@ -157,7 +164,9 @@ QWaylandDisplay::~QWaylandDisplay(void)
         mWaylandIntegration->destroyScreen(screen);
     }
     mScreens.clear();
+#if QT_CONFIG(wayland_datadevice)
     delete mDndSelectionHandler.take();
+#endif
     wl_display_disconnect(mDisplay);
 }
 
@@ -250,8 +259,10 @@ void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uin
     } else if (interface == QStringLiteral("wl_seat")) {
         QWaylandInputDevice *inputDevice = mWaylandIntegration->createInputDevice(this, version, id);
         mInputDevices.append(inputDevice);
+#if QT_CONFIG(wayland_datadevice)
     } else if (interface == QStringLiteral("wl_data_device_manager")) {
         mDndSelectionHandler.reset(new QWaylandDataDeviceManager(this, id));
+#endif
     } else if (interface == QStringLiteral("qt_surface_extension")) {
         mWindowExtension.reset(new QtWayland::qt_surface_extension(registry, id, 1));
     } else if (interface == QStringLiteral("wl_subcompositor")) {
@@ -412,15 +423,24 @@ void QWaylandDisplay::handleKeyboardFocusChanged(QWaylandInputDevice *inputDevic
     if (mLastKeyboardFocus == keyboardFocus)
         return;
 
-    if (keyboardFocus && !keyboardFocus->shellManagesActiveState())
-        handleWindowActivated(keyboardFocus);
-
-    if (mLastKeyboardFocus && !mLastKeyboardFocus->shellManagesActiveState())
-        handleWindowDeactivated(mLastKeyboardFocus);
+    if (mWaylandIntegration->mShellIntegration) {
+        mWaylandIntegration->mShellIntegration->handleKeyboardFocusChanged(keyboardFocus, mLastKeyboardFocus);
+    } else {
+        if (keyboardFocus)
+            handleWindowActivated(keyboardFocus);
+        if (mLastKeyboardFocus)
+            handleWindowDeactivated(mLastKeyboardFocus);
+    }
 
     mLastKeyboardFocus = keyboardFocus;
 }
 
+void QWaylandDisplay::handleWindowDestroyed(QWaylandWindow *window)
+{
+    if (mActiveWindows.contains(window))
+        handleWindowDeactivated(window);
+}
+
 void QWaylandDisplay::handleWaylandSync()
 {
     // This callback is used to set the window activation because we may get an activate/deactivate
@@ -450,6 +470,33 @@ void QWaylandDisplay::requestWaylandSync()
     wl_callback_add_listener(mSyncCallback, &syncCallbackListener, this);
 }
 
+QWaylandInputDevice *QWaylandDisplay::defaultInputDevice() const
+{
+    return mInputDevices.isEmpty() ? 0 : mInputDevices.first();
+}
+
+#if QT_CONFIG(cursor)
+void QWaylandDisplay::setCursor(struct wl_buffer *buffer, struct wl_cursor_image *image)
+{
+    /* Qt doesn't tell us which input device we should set the cursor
+     * for, so set it for all devices. */
+    for (int i = 0; i < mInputDevices.count(); i++) {
+        QWaylandInputDevice *inputDevice = mInputDevices.at(i);
+        inputDevice->setCursor(buffer, image);
+    }
+}
+
+void QWaylandDisplay::setCursor(const QSharedPointer<QWaylandBuffer> &buffer, const QPoint &hotSpot)
+{
+    /* Qt doesn't tell us which input device we should set the cursor
+     * for, so set it for all devices. */
+    for (int i = 0; i < mInputDevices.count(); i++) {
+        QWaylandInputDevice *inputDevice = mInputDevices.at(i);
+        inputDevice->setCursor(buffer, hotSpot);
+    }
+}
+#endif // QT_CONFIG(cursor)
+
 }
 
 QT_END_NAMESPACE
