@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the config.tests of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -39,14 +45,16 @@
 #include "qwaylandinputcontext_p.h"
 #include "qwaylandshmbackingstore_p.h"
 #include "qwaylandnativeinterface_p.h"
+#if QT_CONFIG(clipboard)
 #include "qwaylandclipboard_p.h"
+#endif
 #include "qwaylanddnd_p.h"
 #include "qwaylandwindowmanagerintegration_p.h"
 #include "qwaylandscreen_p.h"
 
-#include "QtPlatformSupport/private/qgenericunixfontdatabase_p.h"
-#include <QtPlatformSupport/private/qgenericunixeventdispatcher_p.h>
-#include <QtPlatformSupport/private/qgenericunixthemes_p.h>
+#include <QtFontDatabaseSupport/private/qgenericunixfontdatabase_p.h>
+#include <QtEventDispatcherSupport/private/qgenericunixeventdispatcher_p.h>
+#include <QtThemeSupport/private/qgenericunixthemes_p.h>
 
 #include <QtGui/private/qguiapplication_p.h>
 
@@ -54,6 +62,7 @@
 #include <qpa/qplatformcursor.h>
 #include <QtGui/QSurfaceFormat>
 #include <QtGui/QOpenGLContext>
+#include <QSocketNotifier>
 
 #include <qpa/qplatforminputcontextfactory_p.h>
 #include <qpa/qplatformaccessibility.h>
@@ -68,6 +77,9 @@
 
 #include "qwaylandshellintegration_p.h"
 #include "qwaylandshellintegrationfactory_p.h"
+#include "qwaylandxdgshellintegration_p.h"
+#include "qwaylandwlshellintegration_p.h"
+#include "qwaylandxdgshellv6integration_p.h"
 
 #include "qwaylandinputdeviceintegration_p.h"
 #include "qwaylandinputdeviceintegrationfactory_p.h"
@@ -87,7 +99,7 @@ public:
             const QByteArray desktopEnvironment = QGuiApplicationPrivate::platformIntegration()->services()->desktopEnvironment();
 
             if (desktopEnvironment == QByteArrayLiteral("KDE")) {
-#ifndef QT_NO_SETTINGS
+#if QT_CONFIG(settings)
                 result.push_back(QStringLiteral("kde"));
 #endif
             } else if (!desktopEnvironment.isEmpty() &&
@@ -110,41 +122,47 @@ public:
 
 QWaylandIntegration::QWaylandIntegration()
     : mClientBufferIntegration(0)
-    , mShellIntegration(Q_NULLPTR)
     , mInputDeviceIntegration(Q_NULLPTR)
     , mFontDb(new QGenericUnixFontDatabase())
     , mNativeInterface(new QWaylandNativeInterface(this))
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
     , mAccessibility(new QPlatformAccessibility())
-#else
-    , mAccessibility(0)
 #endif
     , mClientBufferIntegrationInitialized(false)
     , mServerBufferIntegrationInitialized(false)
     , mShellIntegrationInitialized(false)
 {
     initializeInputDeviceIntegration();
-    mDisplay = new QWaylandDisplay(this);
-    mClipboard = new QWaylandClipboard(mDisplay);
-    mDrag = new QWaylandDrag(mDisplay);
+    mDisplay.reset(new QWaylandDisplay(this));
+#if QT_CONFIG(clipboard)
+    mClipboard.reset(new QWaylandClipboard(mDisplay.data()));
+#endif
+#if QT_CONFIG(draganddrop)
+    mDrag.reset(new QWaylandDrag(mDisplay.data()));
+#endif
+    QString icStr = QPlatformInputContextFactory::requested();
+    if (!icStr.isNull()) {
+        mInputContext.reset(QPlatformInputContextFactory::create(icStr));
+    } else {
+        //try to use the input context using the wl_text_input interface
+        QPlatformInputContext *ctx = new QWaylandInputContext(mDisplay.data());
+        mInputContext.reset(ctx);
 
-    mInputContext.reset(new QWaylandInputContext(mDisplay));
+        //use the traditional way for on screen keyboards for now
+        if (!mInputContext.data()->isValid()) {
+            ctx = QPlatformInputContextFactory::create();
+            mInputContext.reset(ctx);
+        }
+    }
 }
 
 QWaylandIntegration::~QWaylandIntegration()
 {
-    delete mDrag;
-    delete mClipboard;
-#ifndef QT_NO_ACCESSIBILITY
-    delete mAccessibility;
-#endif
-    delete mNativeInterface;
-    delete mDisplay;
 }
 
 QPlatformNativeInterface * QWaylandIntegration::nativeInterface() const
 {
-    return mNativeInterface;
+    return mNativeInterface.data();
 }
 
 bool QWaylandIntegration::hasCapability(QPlatformIntegration::Capability cap) const
@@ -175,14 +193,14 @@ QPlatformWindow *QWaylandIntegration::createPlatformWindow(QWindow *window) cons
     return new QWaylandShmWindow(window);
 }
 
-#ifndef QT_NO_OPENGL
+#if QT_CONFIG(opengl)
 QPlatformOpenGLContext *QWaylandIntegration::createPlatformOpenGLContext(QOpenGLContext *context) const
 {
     if (mDisplay->clientBufferIntegration())
         return mDisplay->clientBufferIntegration()->createPlatformOpenGLContext(context->format(), context->shareHandle());
     return 0;
 }
-#endif // QT_NO_OPENGL
+#endif  // opengl
 
 QPlatformBackingStore *QWaylandIntegration::createPlatformBackingStore(QWindow *window) const
 {
@@ -197,24 +215,37 @@ QAbstractEventDispatcher *QWaylandIntegration::createEventDispatcher() const
 void QWaylandIntegration::initialize()
 {
     QAbstractEventDispatcher *dispatcher = QGuiApplicationPrivate::eventDispatcher;
-    QObject::connect(dispatcher, SIGNAL(aboutToBlock()), mDisplay, SLOT(flushRequests()));
-    QObject::connect(dispatcher, SIGNAL(awake()), mDisplay, SLOT(flushRequests()));
+    QObject::connect(dispatcher, SIGNAL(aboutToBlock()), mDisplay.data(), SLOT(flushRequests()));
+    QObject::connect(dispatcher, SIGNAL(awake()), mDisplay.data(), SLOT(flushRequests()));
+
+    int fd = wl_display_get_fd(mDisplay->wl_display());
+    QSocketNotifier *sn = new QSocketNotifier(fd, QSocketNotifier::Read, mDisplay.data());
+    QObject::connect(sn, SIGNAL(activated(int)), mDisplay.data(), SLOT(flushRequests()));
+
+    if (mDisplay->screens().isEmpty()) {
+        qWarning() << "Running on a compositor with no screens is not supported";
+        ::exit(EXIT_FAILURE);
+    }
 }
 
 QPlatformFontDatabase *QWaylandIntegration::fontDatabase() const
 {
-    return mFontDb;
+    return mFontDb.data();
 }
 
+#if QT_CONFIG(clipboard)
 QPlatformClipboard *QWaylandIntegration::clipboard() const
 {
-    return mClipboard;
+    return mClipboard.data();
 }
+#endif
 
+#if QT_CONFIG(draganddrop)
 QPlatformDrag *QWaylandIntegration::drag() const
 {
-    return mDrag;
+    return mDrag.data();
 }
+#endif  // draganddrop
 
 QPlatformInputContext *QWaylandIntegration::inputContext() const
 {
@@ -236,10 +267,12 @@ QVariant QWaylandIntegration::styleHint(StyleHint hint) const
     return QPlatformIntegration::styleHint(hint);
 }
 
+#if QT_CONFIG(accessibility)
 QPlatformAccessibility *QWaylandIntegration::accessibility() const
 {
-    return mAccessibility;
+    return mAccessibility.data();
 }
+#endif
 
 QPlatformServices *QWaylandIntegration::services() const
 {
@@ -248,7 +281,7 @@ QPlatformServices *QWaylandIntegration::services() const
 
 QWaylandDisplay *QWaylandIntegration::display() const
 {
-    return mDisplay;
+    return mDisplay.data();
 }
 
 QStringList QWaylandIntegration::themeNames() const
@@ -266,7 +299,7 @@ QWaylandClientBufferIntegration *QWaylandIntegration::clientBufferIntegration()
     if (!mClientBufferIntegrationInitialized)
         const_cast<QWaylandIntegration *>(this)->initializeClientBufferIntegration();
 
-    return mClientBufferIntegration && mClientBufferIntegration->isValid() ? mClientBufferIntegration : 0;
+    return mClientBufferIntegration && mClientBufferIntegration->isValid() ? mClientBufferIntegration.data() : nullptr;
 }
 
 QWaylandServerBufferIntegration *QWaylandIntegration::serverBufferIntegration() const
@@ -274,7 +307,7 @@ QWaylandServerBufferIntegration *QWaylandIntegration::serverBufferIntegration()
     if (!mServerBufferIntegrationInitialized)
         const_cast<QWaylandIntegration *>(this)->initializeServerBufferIntegration();
 
-    return mServerBufferIntegration;
+    return mServerBufferIntegration.data();
 }
 
 QWaylandShellIntegration *QWaylandIntegration::shellIntegration() const
@@ -282,7 +315,7 @@ QWaylandShellIntegration *QWaylandIntegration::shellIntegration() const
     if (!mShellIntegrationInitialized)
         const_cast<QWaylandIntegration *>(this)->initializeShellIntegration();
 
-    return mShellIntegration;
+    return mShellIntegration.data();
 }
 
 void QWaylandIntegration::initializeClientBufferIntegration()
@@ -308,10 +341,10 @@ void QWaylandIntegration::initializeClientBufferIntegration()
 
     QStringList keys = QWaylandClientBufferIntegrationFactory::keys();
     if (keys.contains(targetKey)) {
-        mClientBufferIntegration = QWaylandClientBufferIntegrationFactory::create(targetKey, QStringList());
+        mClientBufferIntegration.reset(QWaylandClientBufferIntegrationFactory::create(targetKey, QStringList()));
     }
     if (mClientBufferIntegration)
-        mClientBufferIntegration->initialize(mDisplay);
+        mClientBufferIntegration->initialize(mDisplay.data());
     else
         qWarning("Failed to load client buffer integration: %s\n", qPrintable(targetKey));
 }
@@ -326,7 +359,7 @@ void QWaylandIntegration::initializeServerBufferIntegration()
     disableHardwareIntegration = disableHardwareIntegration || !mDisplay->hardwareIntegration();
     if (disableHardwareIntegration) {
         QByteArray serverBufferIntegrationName = qgetenv("QT_WAYLAND_SERVER_BUFFER_INTEGRATION");
-        QString targetKey = QString::fromLocal8Bit(serverBufferIntegrationName);
+        targetKey = QString::fromLocal8Bit(serverBufferIntegrationName);
     } else {
         targetKey = mDisplay->hardwareIntegration()->serverBufferIntegration();
     }
@@ -338,10 +371,10 @@ void QWaylandIntegration::initializeServerBufferIntegration()
 
     QStringList keys = QWaylandServerBufferIntegrationFactory::keys();
     if (keys.contains(targetKey)) {
-        mServerBufferIntegration = QWaylandServerBufferIntegrationFactory::create(targetKey, QStringList());
+        mServerBufferIntegration.reset(QWaylandServerBufferIntegrationFactory::create(targetKey, QStringList()));
     }
     if (mServerBufferIntegration)
-        mServerBufferIntegration->initialize(mDisplay);
+        mServerBufferIntegration->initialize(mDisplay.data());
     else
         qWarning("Failed to load server buffer integration %s\n", qPrintable(targetKey));
 }
@@ -353,19 +386,30 @@ void QWaylandIntegration::initializeShellIntegration()
     QByteArray integrationName = qgetenv("QT_WAYLAND_SHELL_INTEGRATION");
     QString targetKey = QString::fromLocal8Bit(integrationName);
 
-    if (targetKey.isEmpty()) {
-        return;
+    QStringList preferredShells;
+    if (!targetKey.isEmpty()) {
+        preferredShells << targetKey;
+    } else {
+        preferredShells << QLatin1String("xdg-shell-v6");
+        QString useXdgShell = QString::fromLocal8Bit(qgetenv("QT_WAYLAND_USE_XDG_SHELL"));
+        if (!useXdgShell.isEmpty() && useXdgShell != QLatin1String("0")) {
+            qWarning() << "QT_WAYLAND_USE_XDG_SHELL is deprecated, "
+                          "please specify the shell using QT_WAYLAND_SHELL_INTEGRATION instead";
+            preferredShells << QLatin1String("xdg-shell-v5");
+        }
+        preferredShells << QLatin1String("wl-shell");
     }
 
-    QStringList keys = QWaylandShellIntegrationFactory::keys();
-    if (keys.contains(targetKey)) {
-        mShellIntegration = QWaylandShellIntegrationFactory::create(targetKey, QStringList());
+    Q_FOREACH (QString preferredShell, preferredShells) {
+        mShellIntegration.reset(createShellIntegration(preferredShell));
+        if (mShellIntegration) {
+            qDebug("Using the '%s' shell integration", qPrintable(preferredShell));
+            break;
+        }
     }
-    if (mShellIntegration && mShellIntegration->initialize(mDisplay)) {
-        qDebug("Using the '%s' shell integration", qPrintable(targetKey));
-    } else {
-        delete mShellIntegration;
-        mShellIntegration = Q_NULLPTR;
+
+    if (!mShellIntegration || !mShellIntegration->initialize(mDisplay.data())) {
+        mShellIntegration.reset();
         qWarning("Failed to load shell integration %s", qPrintable(targetKey));
     }
 }
@@ -389,13 +433,28 @@ void QWaylandIntegration::initializeInputDeviceIntegration()
 
     QStringList keys = QWaylandInputDeviceIntegrationFactory::keys();
     if (keys.contains(targetKey)) {
-        mInputDeviceIntegration = QWaylandInputDeviceIntegrationFactory::create(targetKey, QStringList());
+        mInputDeviceIntegration.reset(QWaylandInputDeviceIntegrationFactory::create(targetKey, QStringList()));
         qDebug("Using the '%s' input device integration", qPrintable(targetKey));
     } else {
         qWarning("Wayland inputdevice integration '%s' not found, using default", qPrintable(targetKey));
     }
 }
 
+QWaylandShellIntegration *QWaylandIntegration::createShellIntegration(const QString &integrationName)
+{
+    if (integrationName == QLatin1Literal("wl-shell")) {
+        return QWaylandWlShellIntegration::create(mDisplay.data());
+    } else if (integrationName == QLatin1Literal("xdg-shell-v5")) {
+        return QWaylandXdgShellIntegration::create(mDisplay.data());
+    } else if (integrationName == QLatin1Literal("xdg-shell-v6")) {
+        return QWaylandXdgShellV6Integration::create(mDisplay.data());
+    } else if (QWaylandShellIntegrationFactory::keys().contains(integrationName)) {
+        return QWaylandShellIntegrationFactory::create(integrationName, QStringList());
+    } else {
+        return nullptr;
+    }
+}
+
 }
 
 QT_END_NAMESPACE
