@@ -64,18 +64,24 @@ QWaylandScreen::QWaylandScreen(QWaylandDisplay *waylandDisplay, int version, uin
     , mFormat(QImage::Format_ARGB32_Premultiplied)
     , mOutputName(QStringLiteral("Screen%1").arg(id))
     , m_orientation(Qt::PrimaryOrientation)
+#if QT_CONFIG(cursor)
     , mWaylandCursor(0)
+#endif
 {
 }
 
 QWaylandScreen::~QWaylandScreen()
 {
+#if QT_CONFIG(cursor)
     delete mWaylandCursor;
+#endif
 }
 
 void QWaylandScreen::init()
 {
+#if QT_CONFIG(cursor)
     mWaylandCursor = new QWaylandCursor(this);
+#endif
 }
 
 QWaylandDisplay * QWaylandScreen::display() const
@@ -122,8 +128,10 @@ QList<QPlatformScreen *> QWaylandScreen::virtualSiblings() const
     QList<QPlatformScreen *> list;
     const QList<QWaylandScreen*> screens = mWaylandDisplay->screens();
     list.reserve(screens.count());
-    foreach (QWaylandScreen *screen, screens)
-        list << screen;
+    for (QWaylandScreen *screen : qAsConst(screens)) {
+        if (screen->screen())
+            list << screen;
+    }
     return list;
 }
 
@@ -156,10 +164,12 @@ qreal QWaylandScreen::refreshRate() const
     return mRefreshRate / 1000.f;
 }
 
+#if QT_CONFIG(cursor)
 QPlatformCursor *QWaylandScreen::cursor() const
 {
     return  mWaylandCursor;
 }
+#endif
 
 QWaylandScreen * QWaylandScreen::waylandScreenFromWindow(QWindow *window)
 {
@@ -167,6 +177,12 @@ QWaylandScreen * QWaylandScreen::waylandScreenFromWindow(QWindow *window)
     return static_cast<QWaylandScreen *>(platformScreen);
 }
 
+QWaylandScreen *QWaylandScreen::fromWlOutput(::wl_output *output)
+{
+    auto wlOutput = static_cast<QtWayland::wl_output *>(wl_output_get_user_data(output));
+    return static_cast<QWaylandScreen *>(wlOutput);
+}
+
 void QWaylandScreen::output_mode(uint32_t flags, int width, int height, int refresh)
 {
     if (!(flags & WL_OUTPUT_MODE_CURRENT))
@@ -237,7 +253,7 @@ void QWaylandScreen::output_done()
         QWindowSystemInterface::handleScreenOrientationChange(screen(), m_orientation);
         mTransform = -1;
     }
-    QWindowSystemInterface::handleScreenGeometryChange(screen(), mGeometry, mGeometry);
+    QWindowSystemInterface::handleScreenGeometryChange(screen(), geometry(), geometry());
     QWindowSystemInterface::handleScreenRefreshRateChange(screen(), refreshRate());
 }
 
