@@ -40,7 +40,6 @@
 #include "qwaylandwindow_p.h"
 
 #include "qwaylandbuffer_p.h"
-#include "qwaylanddatadevice_p.h"
 #include "qwaylanddisplay_p.h"
 #include "qwaylandinputdevice_p.h"
 #include "qwaylandscreen_p.h"
@@ -54,6 +53,11 @@
 #include "qwaylanddecorationfactory_p.h"
 #include "qwaylandshmbackingstore_p.h"
 
+#if QT_CONFIG(wayland_datadevice)
+#include "qwaylanddatadevice_p.h"
+#endif
+
+
 #include <QtCore/QFileInfo>
 #include <QtCore/QPointer>
 #include <QtCore/QRegularExpression>
@@ -65,6 +69,8 @@
 
 #include <QtCore/QDebug>
 
+#include <wayland-client-core.h>
+
 QT_BEGIN_NAMESPACE
 
 namespace QtWaylandClient {
@@ -74,21 +80,19 @@ QWaylandWindow *QWaylandWindow::mMouseGrab = 0;
 QWaylandWindow::QWaylandWindow(QWindow *window)
     : QObject()
     , QPlatformWindow(window)
-    , mScreen(QWaylandScreen::waylandScreenFromWindow(window))
-    , mDisplay(mScreen->display())
+    , mDisplay(screen()->display())
     , mShellSurface(0)
     , mSubSurfaceWindow(0)
     , mWindowDecoration(0)
     , mMouseEventsInContentArea(false)
     , mMousePressedInContentArea(Qt::NoButton)
-    , m_cursor(Qt::ArrowCursor)
     , mWaitingForFrameSync(false)
-    , mFrameCallback(0)
     , mRequestResizeSent(false)
     , mCanResize(true)
     , mResizeDirty(false)
     , mResizeAfterSwap(qEnvironmentVariableIsSet("QT_WAYLAND_RESIZE_AFTER_SWAP"))
     , mSentInitialResize(false)
+    , mScale(1)
     , mState(Qt::WindowNoState)
     , mMask()
     , mBackingStore(Q_NULLPTR)
@@ -96,12 +100,14 @@ QWaylandWindow::QWaylandWindow(QWindow *window)
 {
     static WId id = 1;
     mWindowId = id++;
-    if (window->type() != Qt::Desktop)
-        initWindow();
+    connect(qApp, &QGuiApplication::screenRemoved, this, &QWaylandWindow::handleScreenRemoved);
+    initializeWlSurface();
 }
 
 QWaylandWindow::~QWaylandWindow()
 {
+    mDisplay->handleWindowDestroyed(this);
+
     delete mWindowDecoration;
 
     if (isInitialized())
@@ -124,65 +130,73 @@ QWaylandWindow::~QWaylandWindow()
 
 void QWaylandWindow::initWindow()
 {
-    init(mDisplay->createSurface(static_cast<QtWayland::wl_surface *>(this)));
+    if (window()->type() == Qt::Desktop)
+        return;
+
+    if (!isInitialized())
+        initializeWlSurface();
 
     if (shouldCreateSubSurface()) {
+        Q_ASSERT(!mSubSurfaceWindow);
+
         QWaylandWindow *p = static_cast<QWaylandWindow *>(QPlatformWindow::parent());
         if (::wl_subsurface *ss = mDisplay->createSubSurface(this, p)) {
             mSubSurfaceWindow = new QWaylandSubSurface(this, p, ss);
         }
     } else if (shouldCreateShellSurface()) {
+        Q_ASSERT(!mShellSurface);
+
         mShellSurface = mDisplay->createShellSurface(this);
-    }
+        if (mShellSurface) {
+            mShellSurface->setType(window()->type(), transientParent());
 
-    if (mShellSurface) {
-        // Set initial surface title
-        mShellSurface->setTitle(window()->title());
+            // Set initial surface title
+            setWindowTitle(window()->title());
 
-        // The appId is the desktop entry identifier that should follow the
-        // reverse DNS convention (see http://standards.freedesktop.org/desktop-entry-spec/latest/ar01s02.html).
-        // According to xdg-shell the appId is only the name, without
-        // the .desktop suffix.
-        //
-        // If the application specifies the desktop file name use that
-        // removing the ".desktop" suffix, otherwise fall back to the
-        // executable name and prepend the reversed organization domain
-        // when available.
-        if (!QGuiApplication::desktopFileName().isEmpty()) {
-            QString name = QGuiApplication::desktopFileName();
-            mShellSurface->setAppId(name.replace(QRegularExpression(QLatin1String("\\.desktop$")), QString()));
-        } else {
-            QFileInfo fi = QCoreApplication::instance()->applicationFilePath();
-            QStringList domainName =
-                QCoreApplication::instance()->organizationDomain().split(QLatin1Char('.'),
-                                                                         QString::SkipEmptyParts);
-
-            if (domainName.isEmpty()) {
-                mShellSurface->setAppId(fi.baseName());
+            // The appId is the desktop entry identifier that should follow the
+            // reverse DNS convention (see http://standards.freedesktop.org/desktop-entry-spec/latest/ar01s02.html).
+            // According to xdg-shell the appId is only the name, without
+            // the .desktop suffix.
+            //
+            // If the application specifies the desktop file name use that
+            // removing the ".desktop" suffix, otherwise fall back to the
+            // executable name and prepend the reversed organization domain
+            // when available.
+            if (!QGuiApplication::desktopFileName().isEmpty()) {
+                QString name = QGuiApplication::desktopFileName();
+                if (name.endsWith(QLatin1String(".desktop")))
+                    name.chop(8);
+                mShellSurface->setAppId(name);
             } else {
-                QString appId;
-                for (int i = 0; i < domainName.count(); ++i)
-                    appId.prepend(QLatin1Char('.')).prepend(domainName.at(i));
-                appId.append(fi.baseName());
-                mShellSurface->setAppId(appId);
+                QFileInfo fi = QCoreApplication::instance()->applicationFilePath();
+                QStringList domainName =
+                        QCoreApplication::instance()->organizationDomain().split(QLatin1Char('.'),
+                                                                                 QString::SkipEmptyParts);
+
+                if (domainName.isEmpty()) {
+                    mShellSurface->setAppId(fi.baseName());
+                } else {
+                    QString appId;
+                    for (int i = 0; i < domainName.count(); ++i)
+                        appId.prepend(QLatin1Char('.')).prepend(domainName.at(i));
+                    appId.append(fi.baseName());
+                    mShellSurface->setAppId(appId);
+                }
             }
+            // the user may have already set some window properties, so make sure to send them out
+            for (auto it = m_properties.cbegin(); it != m_properties.cend(); ++it)
+                mShellSurface->sendProperty(it.key(), it.value());
+        } else {
+            qWarning("Could not create a shell surface object.");
         }
     }
 
-    if (mShellSurface) {
-        if (window()->transientParent()) {
-            if (window()->type() != Qt::Popup) {
-                mShellSurface->updateTransientParent(window()->transientParent());
-            }
-        } else {
-            if (window()->type() != Qt::ToolTip)
-                mShellSurface->setTopLevel();
-        }
-    }
+    mScale = screen()->scale();
 
     // Enable high-dpi rendering. Scale() returns the screen scale factor and will
     // typically be integer 1 (normal-dpi) or 2 (high-dpi). Call set_buffer_scale()
     // to inform the compositor that high-resolution buffers will be provided.
+    //FIXME this needs to be changed when the screen changes along with a resized backing store
     if (mDisplay->compositorVersion() >= 3)
         set_buffer_scale(scale());
 
@@ -194,8 +208,18 @@ void QWaylandWindow::initWindow()
     else
         setGeometry_helper(window()->geometry());
     setMask(window()->mask());
+    // setWindowStateInternal is a no-op if the argument is equal to mState,
+    // but since we're creating the shellsurface only now we reset mState to
+    // make sure the state gets sent out to the compositor
+    mState = Qt::WindowNoState;
     setWindowStateInternal(window()->windowState());
     handleContentOrientationChange(window()->contentOrientation());
+    mFlags = window()->flags();
+}
+
+void QWaylandWindow::initializeWlSurface()
+{
+    init(mDisplay->createSurface(static_cast<QtWayland::wl_surface *>(this)));
 }
 
 bool QWaylandWindow::shouldCreateShellSurface() const
@@ -223,10 +247,15 @@ void QWaylandWindow::reset()
     mShellSurface = 0;
     delete mSubSurfaceWindow;
     mSubSurfaceWindow = 0;
-    destroy();
+    if (isInitialized())
+        destroy();
 
-    if (mFrameCallback)
+    if (mFrameCallback) {
         wl_callback_destroy(mFrameCallback);
+        mFrameCallback = nullptr;
+    }
+
+    mMask = QRegion();
 }
 
 QWaylandWindow *QWaylandWindow::fromWlSurface(::wl_surface *surface)
@@ -241,6 +270,9 @@ WId QWaylandWindow::winId() const
 
 void QWaylandWindow::setParent(const QPlatformWindow *parent)
 {
+    if (!window()->isVisible())
+        return;
+
     QWaylandWindow *oldparent = mSubSurfaceWindow ? mSubSurfaceWindow->parent() : 0;
     if (oldparent == parent)
         return;
@@ -258,7 +290,8 @@ void QWaylandWindow::setParent(const QPlatformWindow *parent)
 void QWaylandWindow::setWindowTitle(const QString &title)
 {
     if (mShellSurface) {
-        mShellSurface->setTitle(title);
+        const QString separator = QString::fromUtf8(" \xe2\x80\x94 "); // unicode character U+2014, EM DASH
+        mShellSurface->setTitle(formatWindowTitle(title, separator));
     }
 
     if (mWindowDecoration && window()->isVisible())
@@ -282,8 +315,8 @@ void QWaylandWindow::setGeometry_helper(const QRect &rect)
     if (mSubSurfaceWindow) {
         QMargins m = QPlatformWindow::parent()->frameMargins();
         mSubSurfaceWindow->set_position(rect.x() + m.left(), rect.y() + m.top());
-    } else if (shellSurface() && window()->transientParent() && window()->type() != Qt::Popup)
-        shellSurface()->updateTransientParent(window()->transientParent());
+        mSubSurfaceWindow->parent()->window()->requestUpdate();
+    }
 }
 
 void QWaylandWindow::setGeometry(const QRect &rect)
@@ -301,45 +334,59 @@ void QWaylandWindow::setGeometry(const QRect &rect)
 
         mSentInitialResize = true;
     }
+    sendExposeEvent(QRect(QPoint(), geometry().size()));
+}
 
-    QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(), geometry().size()));
+
+
+void QWaylandWindow::sendExposeEvent(const QRect &rect)
+{
+    if (!(mShellSurface && mShellSurface->handleExpose(rect)))
+        QWindowSystemInterface::handleExposeEvent(window(), rect);
+}
+
+
+static QVector<QPointer<QWaylandWindow>> activePopups;
+
+void QWaylandWindow::closePopups(QWaylandWindow *parent)
+{
+    while (!activePopups.isEmpty()) {
+        auto popup = activePopups.takeLast();
+        if (popup.isNull())
+            continue;
+        if (popup.data() == parent)
+            return;
+        popup->reset();
+    }
+}
+
+QWaylandScreen *QWaylandWindow::calculateScreenFromSurfaceEvents() const
+{
+    return mScreens.isEmpty() ? screen() : mScreens.first();
 }
 
 void QWaylandWindow::setVisible(bool visible)
 {
     if (visible) {
-        if (mShellSurface) {
-            if (window()->type() == Qt::Popup) {
-                QWaylandWindow *parent = transientParent();
-                if (parent) {
-                    QWaylandWlShellSurface *wlshellSurface = qobject_cast<QWaylandWlShellSurface*>(mShellSurface);
-                    if (wlshellSurface)
-                        wlshellSurface->setPopup(parent, mDisplay->lastInputDevice(), mDisplay->lastInputSerial());
-                }
-            } else if (window()->type() == Qt::ToolTip) {
-                if (QWaylandWindow *parent = transientParent()) {
-                    mShellSurface->updateTransientParent(parent->window());
-                }
-            }
-        }
+        if (window()->type() == Qt::Popup)
+            activePopups << this;
+        initWindow();
+        mDisplay->flushRequests();
 
         setGeometry(window()->geometry());
         // Don't flush the events here, or else the newly visible window may start drawing, but since
         // there was no frame before it will be stuck at the waitForFrameSync() in
         // QWaylandShmBackingStore::beginPaint().
     } else {
-        QWindowSystemInterface::handleExposeEvent(window(), QRegion());
+        sendExposeEvent(QRect());
         // when flushing the event queue, it could contain a close event, in which
         // case 'this' will be deleted. When that happens, we must abort right away.
         QPointer<QWaylandWindow> deleteGuard(this);
         QWindowSystemInterface::flushWindowSystemEvents();
-        if (!deleteGuard.isNull()) {
-            attach(static_cast<QWaylandBuffer *>(0), 0, 0);
-            commit();
-            if (mBackingStore) {
-                mBackingStore->hidden();
-            }
-        }
+        if (!deleteGuard.isNull() && window()->type() == Qt::Popup)
+            closePopups(this);
+        if (!deleteGuard.isNull())
+            reset();
     }
 }
 
@@ -364,15 +411,18 @@ void QWaylandWindow::setMask(const QRegion &mask)
 
     mMask = mask;
 
+    if (!isInitialized())
+        return;
+
     if (mMask.isEmpty()) {
-        set_input_region(0);
+        set_input_region(nullptr);
     } else {
         struct ::wl_region *region = mDisplay->createRegion(mMask);
         set_input_region(region);
         wl_region_destroy(region);
     }
 
-    commit();
+    wl_surface::commit();
 }
 
 void QWaylandWindow::configure(uint32_t edges, int32_t width, int32_t height)
@@ -428,10 +478,10 @@ void QWaylandWindow::setCanResize(bool canResize)
         }
         if (!mConfigure.isEmpty()) {
             doResize();
-            QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(), geometry().size()));
+            sendExposeEvent(QRect(QPoint(), geometry().size()));
         } else if (mResizeDirty) {
             mResizeDirty = false;
-            QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(), geometry().size()));
+            sendExposeEvent(QRect(QPoint(), geometry().size()));
         }
     }
 }
@@ -446,21 +496,69 @@ void QWaylandWindow::requestResize()
 
     mRequestResizeSent = false;
     lock.unlock();
-    QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(), geometry().size()));
+    sendExposeEvent(QRect(QPoint(), geometry().size()));
     QWindowSystemInterface::flushWindowSystemEvents();
 }
 
+void QWaylandWindow::surface_enter(wl_output *output)
+{
+    QWaylandScreen *oldScreen = calculateScreenFromSurfaceEvents();
+    auto addedScreen = QWaylandScreen::fromWlOutput(output);
+
+    if (mScreens.contains(addedScreen)) {
+        qWarning() << "Unexpected wl_surface.enter received for output with id:"
+                   << wl_proxy_get_id(reinterpret_cast<wl_proxy *>(output))
+                   << "screen name:" << addedScreen->name() << "screen model:" << addedScreen->model();
+        return;
+    }
+
+    mScreens.append(addedScreen);
+
+    QWaylandScreen *newScreen = calculateScreenFromSurfaceEvents();
+    if (oldScreen != newScreen) //currently this will only happen if the first wl_surface.enter is for a non-primary screen
+        QWindowSystemInterface::handleWindowScreenChanged(window(), newScreen->QPlatformScreen::screen());
+}
+
+void QWaylandWindow::surface_leave(wl_output *output)
+{
+    QWaylandScreen *oldScreen = calculateScreenFromSurfaceEvents();
+    auto *removedScreen = QWaylandScreen::fromWlOutput(output);
+    bool wasRemoved = mScreens.removeOne(removedScreen);
+    if (!wasRemoved) {
+        qWarning() << "Unexpected wl_surface.leave received for output with id:"
+                   << wl_proxy_get_id(reinterpret_cast<wl_proxy *>(output))
+                   << "screen name:" << removedScreen->name() << "screen model:" << removedScreen->model();
+        return;
+    }
+
+    QWaylandScreen *newScreen = calculateScreenFromSurfaceEvents();
+    if (oldScreen != newScreen)
+        QWindowSystemInterface::handleWindowScreenChanged(window(), newScreen->QPlatformScreen::screen());
+}
+
+void QWaylandWindow::handleScreenRemoved(QScreen *qScreen)
+{
+    QWaylandScreen *oldScreen = calculateScreenFromSurfaceEvents();
+    bool wasRemoved = mScreens.removeOne(static_cast<QWaylandScreen *>(qScreen->handle()));
+    if (wasRemoved) {
+        QWaylandScreen *newScreen = calculateScreenFromSurfaceEvents();
+        if (oldScreen != newScreen)
+            QWindowSystemInterface::handleWindowScreenChanged(window(), newScreen->QPlatformScreen::screen());
+    }
+}
+
 void QWaylandWindow::attach(QWaylandBuffer *buffer, int x, int y)
 {
     if (mFrameCallback) {
         wl_callback_destroy(mFrameCallback);
-        mFrameCallback = 0;
+        mFrameCallback = nullptr;
     }
 
     if (buffer) {
         mFrameCallback = frame();
         wl_callback_add_listener(mFrameCallback, &QWaylandWindow::callbackListener, this);
         mWaitingForFrameSync = true;
+        buffer->setBusy();
 
         attach(buffer->buffer(), x, y);
     } else {
@@ -479,6 +577,18 @@ void QWaylandWindow::damage(const QRect &rect)
     damage(rect.x(), rect.y(), rect.width(), rect.height());
 }
 
+void QWaylandWindow::commit(QWaylandBuffer *buffer, const QRegion &damage)
+{
+    if (!isInitialized())
+        return;
+
+    attachOffset(buffer);
+    const QVector<QRect> rects = damage.rects();
+    for (const QRect &rect: rects)
+        wl_surface::damage(rect.x(), rect.y(), rect.width(), rect.height());
+    wl_surface::commit();
+}
+
 const wl_callback_listener QWaylandWindow::callbackListener = {
     QWaylandWindow::frameCallback
 };
@@ -489,12 +599,10 @@ void QWaylandWindow::frameCallback(void *data, struct wl_callback *callback, uin
     QWaylandWindow *self = static_cast<QWaylandWindow*>(data);
 
     self->mWaitingForFrameSync = false;
-    wl_callback_destroy(callback);
-    self->mFrameCallback = 0;
     if (self->mUpdateRequested) {
         QWindowPrivate *w = QWindowPrivate::get(self->window());
-        w->deliverUpdateRequest();
         self->mUpdateRequested = false;
+        w->deliverUpdateRequest();
     }
 }
 
@@ -527,9 +635,9 @@ QWaylandSubSurface *QWaylandWindow::subSurfaceWindow() const
     return mSubSurfaceWindow;
 }
 
-bool QWaylandWindow::shellManagesActiveState() const
+QWaylandScreen *QWaylandWindow::screen() const
 {
-    return mShellSurface && mShellSurface->shellManagesActiveState();
+    return static_cast<QWaylandScreen *>(QPlatformWindow::screen());
 }
 
 void QWaylandWindow::handleContentOrientationChange(Qt::ScreenOrientation orientation)
@@ -560,7 +668,7 @@ void QWaylandWindow::handleContentOrientationChange(Qt::ScreenOrientation orient
     }
     set_buffer_transform(transform);
     // set_buffer_transform is double buffered, we need to commit.
-    commit();
+    wl_surface::commit();
 }
 
 void QWaylandWindow::setOrientationMask(Qt::ScreenOrientations mask)
@@ -579,6 +687,9 @@ void QWaylandWindow::setWindowFlags(Qt::WindowFlags flags)
 {
     if (mShellSurface)
         mShellSurface->setWindowFlags(flags);
+
+    mFlags = flags;
+    createDecoration();
 }
 
 bool QWaylandWindow::createDecoration()
@@ -613,9 +724,9 @@ bool QWaylandWindow::createDecoration()
         default:
             break;
     }
-    if (window()->flags() & Qt::FramelessWindowHint || isFullscreen())
+    if (mFlags & Qt::FramelessWindowHint || isFullscreen())
         decoration = false;
-    if (window()->flags() & Qt::BypassWindowManagerHint)
+    if (mFlags & Qt::BypassWindowManagerHint)
         decoration = false;
     if (mSubSurfaceWindow)
         decoration = false;
@@ -663,6 +774,7 @@ bool QWaylandWindow::createDecoration()
             QMargins m = frameMargins();
             subsurf->set_position(pos.x() + m.left(), pos.y() + m.top());
         }
+        sendExposeEvent(QRect(QPoint(), geometry().size()));
     }
 
     return mWindowDecoration;
@@ -673,24 +785,29 @@ QWaylandAbstractDecoration *QWaylandWindow::decoration() const
     return mWindowDecoration;
 }
 
-static QWindow *topLevelWindow(QWindow *window)
+static QWaylandWindow *closestShellSurfaceWindow(QWindow *window)
 {
-    while (QWindow *parent = window->parent())
-        window = parent;
-    return window;
+    while (window) {
+        auto w = static_cast<QWaylandWindow *>(window->handle());
+        if (w && w->shellSurface())
+            return w;
+        window = window->transientParent() ? window->transientParent() : window->parent();
+    }
+    return nullptr;
 }
 
 QWaylandWindow *QWaylandWindow::transientParent() const
 {
-    if (window()->transientParent()) {
-        // Take the top level window here, since the transient parent may be a QWidgetWindow
-        // or some other window without a shell surface, which is then not able to get mouse
-        // events.
-        return static_cast<QWaylandWindow *>(topLevelWindow(window()->transientParent())->handle());
-    }
-    // Try with the current focus window. It should be the right one and anyway
-    // better than having no parent at all.
-    return mDisplay->lastInputWindow();
+    // Take the closest window with a shell surface, since the transient parent may be a
+    // QWidgetWindow or some other window without a shell surface, which is then not able to
+    // get mouse events.
+    if (auto transientParent = closestShellSurfaceWindow(window()->transientParent()))
+        return transientParent;
+
+    if (QGuiApplication::focusWindow() && (window()->type() == Qt::ToolTip || window()->type() == Qt::Popup))
+        return closestShellSurfaceWindow(QGuiApplication::focusWindow());
+
+    return nullptr;
 }
 
 void QWaylandWindow::handleMouse(QWaylandInputDevice *inputDevice, const QWaylandPointerEvent &e)
@@ -707,13 +824,15 @@ void QWaylandWindow::handleMouse(QWaylandInputDevice *inputDevice, const QWaylan
                 QWindowSystemInterface::handleMouseEvent(window(), e.timestamp, e.local, e.global, e.buttons, e.modifiers);
                 break;
             case QWaylandPointerEvent::Wheel:
-                QWindowSystemInterface::handleWheelEvent(window(), e.timestamp, e.local, e.global, e.pixelDelta, e.angleDelta);
+                QWindowSystemInterface::handleWheelEvent(window(), e.timestamp, e.local, e.global, e.pixelDelta, e.angleDelta, e.modifiers);
                 break;
         }
     }
 
+#if QT_CONFIG(cursor)
     if (e.type == QWaylandPointerEvent::Enter)
         restoreMouseCursor(inputDevice);
+#endif
 }
 
 void QWaylandWindow::handleMouseLeave(QWaylandInputDevice *inputDevice)
@@ -725,7 +844,9 @@ void QWaylandWindow::handleMouseLeave(QWaylandInputDevice *inputDevice)
     } else {
         QWindowSystemInterface::handleLeaveEvent(window());
     }
+#if QT_CONFIG(cursor)
     restoreMouseCursor(inputDevice);
+#endif
 }
 
 bool QWaylandWindow::touchDragDecoration(QWaylandInputDevice *inputDevice, const QPointF &local, const QPointF &global, Qt::TouchPointState state, Qt::KeyboardModifiers mods)
@@ -757,7 +878,9 @@ void QWaylandWindow::handleMouseEventWithDecoration(QWaylandInputDevice *inputDe
         globalTranslated.setX(globalTranslated.x() - marg.left());
         globalTranslated.setY(globalTranslated.y() - marg.top());
         if (!mMouseEventsInContentArea) {
+#if QT_CONFIG(cursor)
             restoreMouseCursor(inputDevice);
+#endif
             QWindowSystemInterface::handleEnterEvent(window());
         }
 
@@ -769,7 +892,7 @@ void QWaylandWindow::handleMouseEventWithDecoration(QWaylandInputDevice *inputDe
                 QWindowSystemInterface::handleMouseEvent(window(), e.timestamp, localTranslated, globalTranslated, e.buttons, e.modifiers);
                 break;
             case QWaylandPointerEvent::Wheel:
-                QWindowSystemInterface::handleWheelEvent(window(), e.timestamp, localTranslated, globalTranslated, e.pixelDelta, e.angleDelta);
+                QWindowSystemInterface::handleWheelEvent(window(), e.timestamp, localTranslated, globalTranslated, e.pixelDelta, e.angleDelta, e.modifiers);
                 break;
         }
 
@@ -783,18 +906,17 @@ void QWaylandWindow::handleMouseEventWithDecoration(QWaylandInputDevice *inputDe
     }
 }
 
+#if QT_CONFIG(cursor)
 void QWaylandWindow::setMouseCursor(QWaylandInputDevice *device, const QCursor &cursor)
 {
-    if (device->serial() >= device->cursorSerial()) {
-        device->setCursor(cursor, mScreen);
-        m_cursor = cursor;
-    }
+    device->setCursor(cursor, screen());
 }
 
 void QWaylandWindow::restoreMouseCursor(QWaylandInputDevice *device)
 {
     setMouseCursor(device, window()->cursor());
 }
+#endif
 
 void QWaylandWindow::requestActivateWindow()
 {
@@ -804,10 +926,12 @@ void QWaylandWindow::requestActivateWindow()
 
 void QWaylandWindow::unfocus()
 {
+#if QT_CONFIG(clipboard)
     QWaylandInputDevice *inputDevice = mDisplay->currentInputDevice();
     if (inputDevice && inputDevice->dataDevice()) {
         inputDevice->dataDevice()->invalidateSelectionOffer();
     }
+#endif
 }
 
 bool QWaylandWindow::isExposed() const
@@ -819,12 +943,12 @@ bool QWaylandWindow::isExposed() const
 
 int QWaylandWindow::scale() const
 {
-    return screen()->scale();
+    return mScale;
 }
 
 qreal QWaylandWindow::devicePixelRatio() const
 {
-    return screen()->devicePixelRatio();
+    return mScale;
 }
 
 bool QWaylandWindow::setMouseGrabEnabled(bool grab)
@@ -905,7 +1029,7 @@ QVariant QWaylandWindow::property(const QString &name, const QVariant &defaultVa
 
 void QWaylandWindow::requestUpdate()
 {
-    if (!mFrameCallback)
+    if (!mWaitingForFrameSync)
         QPlatformWindow::requestUpdate();
     else
         mUpdateRequested = true;
