@@ -59,7 +59,7 @@
 #include <qpa/qplatformwindow.h>
 
 #include <QtWaylandClient/private/qwayland-wayland.h>
-#include <QtWaylandClient/qwaylandclientexport.h>
+#include <QtWaylandClient/qtwaylandclientglobal.h>
 
 struct wl_egl_window;
 
@@ -110,16 +110,16 @@ public:
     ~QWaylandWindow();
 
     virtual WindowType windowType() const = 0;
-    WId winId() const Q_DECL_OVERRIDE;
-    void setVisible(bool visible) Q_DECL_OVERRIDE;
-    void setParent(const QPlatformWindow *parent) Q_DECL_OVERRIDE;
+    WId winId() const override;
+    void setVisible(bool visible) override;
+    void setParent(const QPlatformWindow *parent) override;
 
-    void setWindowTitle(const QString &title) Q_DECL_OVERRIDE;
+    void setWindowTitle(const QString &title) override;
 
     inline QIcon windowIcon() const;
-    void setWindowIcon(const QIcon &icon) Q_DECL_OVERRIDE;
+    void setWindowIcon(const QIcon &icon) override;
 
-    void setGeometry(const QRect &rect) Q_DECL_OVERRIDE;
+    void setGeometry(const QRect &rect) override;
 
     void configure(uint32_t edges, int32_t width, int32_t height);
 
@@ -131,35 +131,35 @@ public:
     using QtWayland::wl_surface::damage;
     void damage(const QRect &rect);
 
+    void commit(QWaylandBuffer *buffer, const QRegion &damage);
+
     void waitForFrameSync();
 
-    QMargins frameMargins() const Q_DECL_OVERRIDE;
+    QMargins frameMargins() const override;
 
     static QWaylandWindow *fromWlSurface(::wl_surface *surface);
 
     QWaylandDisplay *display() const { return mDisplay; }
     QWaylandShellSurface *shellSurface() const;
     QWaylandSubSurface *subSurfaceWindow() const;
-    QWaylandScreen *screen() const { return mScreen; }
+    QWaylandScreen *screen() const;
 
-    bool shellManagesActiveState() const;
-
-    void handleContentOrientationChange(Qt::ScreenOrientation orientation) Q_DECL_OVERRIDE;
+    void handleContentOrientationChange(Qt::ScreenOrientation orientation) override;
     void setOrientationMask(Qt::ScreenOrientations mask);
 
-    void setWindowState(Qt::WindowState state) Q_DECL_OVERRIDE;
-    void setWindowFlags(Qt::WindowFlags flags) Q_DECL_OVERRIDE;
+    void setWindowState(Qt::WindowState state) override;
+    void setWindowFlags(Qt::WindowFlags flags) override;
 
-    void raise() Q_DECL_OVERRIDE;
-    void lower() Q_DECL_OVERRIDE;
+    void raise() override;
+    void lower() override;
 
-    void setMask(const QRegion &region) Q_DECL_OVERRIDE;
+    void setMask(const QRegion &region) override;
 
     int scale() const;
-    qreal devicePixelRatio() const Q_DECL_OVERRIDE;
+    qreal devicePixelRatio() const override;
 
-    void requestActivateWindow() Q_DECL_OVERRIDE;
-    bool isExposed() const Q_DECL_OVERRIDE;
+    void requestActivateWindow() override;
+    bool isExposed() const override;
     void unfocus();
 
     QWaylandAbstractDecoration *decoration() const;
@@ -175,8 +175,10 @@ public:
     inline bool isMaximized() const { return mState == Qt::WindowMaximized; }
     inline bool isFullscreen() const { return mState == Qt::WindowFullScreen; }
 
+#if QT_CONFIG(cursor)
     void setMouseCursor(QWaylandInputDevice *device, const QCursor &cursor);
     void restoreMouseCursor(QWaylandInputDevice *device);
+#endif
 
     QWaylandWindow *transientParent() const;
 
@@ -184,7 +186,7 @@ public:
     void doResize();
     void setCanResize(bool canResize);
 
-    bool setMouseGrabEnabled(bool grab) Q_DECL_OVERRIDE;
+    bool setMouseGrabEnabled(bool grab) override;
     static QWaylandWindow *mouseGrab() { return mMouseGrab; }
 
     void sendProperty(const QString &name, const QVariant &value);
@@ -197,17 +199,20 @@ public:
     void setBackingStore(QWaylandShmBackingStore *backingStore) { mBackingStore = backingStore; }
     QWaylandShmBackingStore *backingStore() const { return mBackingStore; }
 
-    bool setKeyboardGrabEnabled(bool) Q_DECL_OVERRIDE { return false; }
-    void propagateSizeHints() Q_DECL_OVERRIDE { }
+    bool setKeyboardGrabEnabled(bool) override { return false; }
+    void propagateSizeHints() override { }
     void addAttachOffset(const QPoint point);
 
-    void requestUpdate() Q_DECL_OVERRIDE;
+    void requestUpdate() override;
 
 public slots:
     void requestResize();
 
 protected:
-    QWaylandScreen *mScreen;
+    void surface_enter(struct ::wl_output *output) override;
+    void surface_leave(struct ::wl_output *output) override;
+
+    QVector<QWaylandScreen *> mScreens; //As seen by wl_surface.enter/leave events. Chronological order.
     QWaylandDisplay *mDisplay;
     QWaylandShellSurface *mShellSurface;
     QWaylandSubSurface *mSubSurfaceWindow;
@@ -216,11 +221,10 @@ protected:
     QWaylandAbstractDecoration *mWindowDecoration;
     bool mMouseEventsInContentArea;
     Qt::MouseButtons mMousePressedInContentArea;
-    QCursor m_cursor;
 
     WId mWindowId;
     bool mWaitingForFrameSync;
-    struct wl_callback *mFrameCallback;
+    struct ::wl_callback *mFrameCallback = nullptr;
     QWaitCondition mFrameSyncWait;
 
     QMutex mResizeLock;
@@ -233,21 +237,30 @@ protected:
 
     bool mSentInitialResize;
     QPoint mOffset;
+    int mScale;
 
     QIcon mWindowIcon;
 
     Qt::WindowState mState;
+    Qt::WindowFlags mFlags;
     QRegion mMask;
 
     QWaylandShmBackingStore *mBackingStore;
 
+private slots:
+    void handleScreenRemoved(QScreen *qScreen);
+
 private:
     bool setWindowStateInternal(Qt::WindowState flags);
     void setGeometry_helper(const QRect &rect);
     void initWindow();
+    void initializeWlSurface();
     bool shouldCreateShellSurface() const;
     bool shouldCreateSubSurface() const;
     void reset();
+    void sendExposeEvent(const QRect &rect);
+    static void closePopups(QWaylandWindow *parent);
+    QWaylandScreen *calculateScreenFromSurfaceEvents() const;
 
     void handleMouseEventWithDecoration(QWaylandInputDevice *inputDevice, const QWaylandPointerEvent &e);
 
