@@ -1,34 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Jolla Ltd, author: <giulio.camuffo@jollamobile.com>
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2017 Jolla Ltd, author: <giulio.camuffo@jollamobile.com>
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -38,14 +41,14 @@
 #include <QAtomicInt>
 
 #include "qwaylandbufferref.h"
-#include "wayland_wrapper/qwlsurfacebuffer_p.h"
+#include "wayland_wrapper/qwlclientbuffer_p.h"
 
 QT_BEGIN_NAMESPACE
 
 class QWaylandBufferRefPrivate
 {
 public:
-    QtWayland::SurfaceBuffer *buffer;
+    QtWayland::ClientBuffer *buffer;
 
     bool nullOrDestroyed() {
         return !buffer || buffer->isDestroyed();
@@ -55,7 +58,7 @@ public:
 /*!
  * \class QWaylandBufferRef
  * \inmodule QtWaylandCompositor
- * \preliminary
+ * \since 5.8
  * \brief The QWaylandBufferRef class holds the reference to a surface buffer
  *
  * This class can be used to reference a surface buffer. As long as a reference
@@ -74,7 +77,7 @@ QWaylandBufferRef::QWaylandBufferRef()
 /*!
  * Constructs a reference to \a buffer.
  */
-QWaylandBufferRef::QWaylandBufferRef(QtWayland::SurfaceBuffer *buffer)
+QWaylandBufferRef::QWaylandBufferRef(QtWayland::ClientBuffer *buffer)
                  : d(new QWaylandBufferRefPrivate)
 {
     d->buffer = buffer;
@@ -104,17 +107,18 @@ QWaylandBufferRef::~QWaylandBufferRef()
 }
 
 /*!
- * Assigns \a ref to this buffer. The previously referenced buffer is
- * dereferenced and the new one gets a new reference.
+ * Assigns \a ref to this buffer and adds a reference to it. The previously referenced buffer is
+ * dereferenced.
  */
 QWaylandBufferRef &QWaylandBufferRef::operator=(const QWaylandBufferRef &ref)
 {
+    if (ref.d->buffer)
+        ref.d->buffer->ref();
+
     if (d->buffer)
         d->buffer->deref();
 
     d->buffer = ref.d->buffer;
-    if (d->buffer)
-        d->buffer->ref();
 
     return *this;
 }
@@ -141,7 +145,7 @@ bool QWaylandBufferRef::operator!=(const QWaylandBufferRef &ref)
  * Returns true if this QWaylandBufferRef does not reference a buffer.
  * Otherwise returns false.
  *
- * \sa hasBuffer()
+ * \sa hasBuffer(), hasContent()
  */
 bool QWaylandBufferRef::isNull() const
 {
@@ -151,12 +155,21 @@ bool QWaylandBufferRef::isNull() const
 /*!
  * Returns true if this QWaylandBufferRef references a buffer. Otherwise returns false.
  *
- * \sa isNull()
+ * \sa isNull(), hasContent()
  */
 bool QWaylandBufferRef::hasBuffer() const
 {
     return d->buffer;
 }
+/*!
+ * Returns true if this QWaylandBufferRef references a buffer that has content. Otherwise returns false.
+ *
+ * \sa isNull(), hasBuffer()
+ */
+bool QWaylandBufferRef::hasContent() const
+{
+    return QtWayland::ClientBuffer::hasContent(d->buffer);
+}
 
 /*!
  * Returns true if this QWaylandBufferRef references a buffer that
@@ -175,6 +188,14 @@ struct ::wl_resource *QWaylandBufferRef::wl_buffer() const
     return d->buffer ? d->buffer->waylandBufferHandle() : Q_NULLPTR;
 }
 
+/*!
+ * \internal
+ */
+QtWayland::ClientBuffer *QWaylandBufferRef::buffer() const
+{
+    return d->buffer;
+}
+
 /*!
  * Returns the size of the buffer.
  * If the buffer referenced is null, an invalid QSize() is returned.
@@ -205,8 +226,8 @@ QWaylandBufferRef::BufferType QWaylandBufferRef::bufferType() const
     if (d->nullOrDestroyed())
         return BufferType_Null;
 
-    if (isShm())
-        return BufferType_Shm;
+    if (isSharedMemory())
+        return BufferType_SharedMemory;
 
     return BufferType_Egl;
 }
@@ -222,12 +243,12 @@ QWaylandBufferRef::BufferFormatEgl QWaylandBufferRef::bufferFormatEgl() const
 /*!
  * Returns true if the buffer is a shared memory buffer. Otherwise returns false.
  */
-bool QWaylandBufferRef::isShm() const
+bool QWaylandBufferRef::isSharedMemory() const
 {
     if (d->nullOrDestroyed())
         return false;
 
-    return d->buffer->isShm();
+    return d->buffer->isSharedMemory();
 }
 
 /*!
@@ -241,36 +262,44 @@ QImage QWaylandBufferRef::image() const
     return d->buffer->image();
 }
 
-#ifdef QT_COMPOSITOR_WAYLAND_GL
-GLuint QWaylandBufferRef::textureForPlane(int plane) const
+#if QT_CONFIG(opengl)
+/*!
+ * Returns an OpenGL texture for the buffer. \a plane is the index for multi-plane formats, such as YUV.
+ *
+ * The returned texture is owned by the buffer. The texture is only valid for as
+ * long as the buffer reference exists. The caller of this function must not delete the texture, and must
+ * keep a reference to the buffer for as long as the texture is being used.
+ *
+ * Returns \c nullptr if there is no valid buffer, or if no texture can be created.
+ */
+QOpenGLTexture *QWaylandBufferRef::toOpenGLTexture(int plane) const
 {
     if (d->nullOrDestroyed())
-        return 0;
+        return nullptr;
 
-    return d->buffer->textureForPlane(plane);
+    return d->buffer->toOpenGlTexture(plane);
 }
-#endif
 
 /*!
- * Binds the buffer to the current OpenGL texture. This may
- * perform a copy of the buffer data, depending on the platform
- * and the type of the buffer.
+ * Returns the native handle for this buffer, and marks it as locked so it will not be
+ * released until unlockNativeBuffer() is called.
+ *
+ * Returns 0 if there is no native handle for this buffer, or if the lock was unsuccessful.
  */
-void QWaylandBufferRef::bindToTexture() const
+quintptr QWaylandBufferRef::lockNativeBuffer()
 {
-    if (d->nullOrDestroyed())
-        return;
-
-    return d->buffer->bindToTexture();
-
+    return d->buffer->lockNativeBuffer();
 }
 
-void QWaylandBufferRef::updateTexture() const
+/*!
+ * Marks the native buffer as no longer in use. \a handle must correspond to the value returned by
+ * a previous call to lockNativeBuffer().
+ */
+void QWaylandBufferRef::unlockNativeBuffer(quintptr handle)
 {
-    if (d->nullOrDestroyed() || d->buffer->isShm())
-        return;
-
-    d->buffer->updateTexture();
+    d->buffer->unlockNativeBuffer(handle);
 }
 
+#endif
+
 QT_END_NAMESPACE
