@@ -1,35 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2014-2015 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
-** Copyright (C) 2013 Klarälvdalens Datakonsult AB (KDAB).
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2017-2016 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
+** Copyright (C) 2017 Klarälvdalens Datakonsult AB (KDAB).
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -39,6 +42,7 @@
 #define QWAYLANDOUTPUT_H
 
 #include <QtWaylandCompositor/qwaylandcompositorextension.h>
+#include <QtWaylandCompositor/QWaylandOutputMode>
 #include <QtCore/QObject>
 
 #include <QObject>
@@ -66,7 +70,6 @@ class Q_WAYLAND_COMPOSITOR_EXPORT QWaylandOutput : public QWaylandObject
     Q_PROPERTY(QString manufacturer READ manufacturer WRITE setManufacturer NOTIFY manufacturerChanged)
     Q_PROPERTY(QString model READ model WRITE setModel NOTIFY modelChanged)
     Q_PROPERTY(QPoint position READ position WRITE setPosition NOTIFY positionChanged)
-    Q_PROPERTY(QWaylandOutput::Mode mode READ mode WRITE setMode NOTIFY modeChanged)
     Q_PROPERTY(QRect geometry READ geometry NOTIFY geometryChanged)
     Q_PROPERTY(QRect availableGeometry READ availableGeometry WRITE setAvailableGeometry NOTIFY availableGeometryChanged)
     Q_PROPERTY(QSize physicalSize READ physicalSize WRITE setPhysicalSize NOTIFY physicalSizeChanged)
@@ -99,12 +102,6 @@ public:
     };
     Q_ENUM(Transform)
 
-    struct Mode
-    {
-        QSize size;
-        qreal refreshRate;
-    };
-
     QWaylandOutput();
     QWaylandOutput(QWaylandCompositor *compositor, QWindow *window);
     ~QWaylandOutput();
@@ -127,13 +124,14 @@ public:
     QPoint position() const;
     void setPosition(const QPoint &pt);
 
-    Mode mode() const;
-    void setMode(const Mode &mode);
+    QList<QWaylandOutputMode> modes() const;
+
+    void addMode(const QWaylandOutputMode &mode, bool preferred = false);
+
+    QWaylandOutputMode currentMode() const;
+    void setCurrentMode(const QWaylandOutputMode &mode);
 
     QRect geometry() const;
-    void setGeometry(const QRect &geometry);
-    void setWidth(int newWidth);
-    void setHeight(int newHeight);
 
     QRect availableGeometry() const;
     void setAvailableGeometry(const QRect &availableGeometry);
@@ -169,7 +167,8 @@ Q_SIGNALS:
     void windowChanged();
     void positionChanged();
     void geometryChanged();
-    void modeChanged();
+    void modeAdded();
+    void currentModeChanged();
     void availableGeometryChanged();
     void physicalSizeChanged();
     void scaleFactorChanged();
@@ -182,16 +181,16 @@ Q_SIGNALS:
     void windowDestroyed();
 
 private Q_SLOTS:
+    void handleSetWidth(int newWidth);
+    void handleSetHeight(int newHeight);
     void handleWindowDestroyed();
 
 protected:
-    bool event(QEvent *event) Q_DECL_OVERRIDE;
+    bool event(QEvent *event) override;
 
     virtual void initialize();
 };
 
 QT_END_NAMESPACE
 
-Q_DECLARE_METATYPE(QWaylandOutput::Mode)
-
 #endif // QWAYLANDOUTPUT_H
