@@ -1,35 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2017 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
+** Copyright (C) 2017 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -37,6 +40,7 @@
 
 #include "qwaylandquickoutput.h"
 #include "qwaylandquickcompositor.h"
+#include "qwaylandquickitem_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -71,6 +75,22 @@ void QWaylandQuickOutput::initialize()
             this, &QWaylandQuickOutput::doFrameCallbacks);
 }
 
+void QWaylandQuickOutput::classBegin()
+{
+}
+
+void QWaylandQuickOutput::componentComplete()
+{
+    if (!compositor()) {
+        for (QObject *p = parent(); p != nullptr; p = p->parent()) {
+            if (auto c = qobject_cast<QWaylandCompositor *>(p)) {
+                setCompositor(c);
+                break;
+            }
+        }
+    }
+}
+
 void QWaylandQuickOutput::update()
 {
     if (!m_updateScheduled) {
@@ -102,6 +122,44 @@ void QWaylandQuickOutput::setAutomaticFrameCallback(bool automatic)
     automaticFrameCallbackChanged();
 }
 
+static QQuickItem* clickableItemAtPosition(QQuickItem *rootItem, const QPointF &position)
+{
+    if (!rootItem->isEnabled() || !rootItem->isVisible())
+        return nullptr;
+
+    QList<QQuickItem *> paintOrderItems = QQuickItemPrivate::get(rootItem)->paintOrderChildItems();
+    auto negativeZStart = paintOrderItems.crend();
+    for (auto it = paintOrderItems.crbegin(); it != paintOrderItems.crend(); ++it) {
+        if ((*it)->z() < 0) {
+            negativeZStart = it;
+            break;
+        }
+        QQuickItem *item = clickableItemAtPosition(*it, rootItem->mapToItem(*it, position));
+        if (item)
+            return item;
+    }
+
+    if (rootItem->contains(position) && rootItem->acceptedMouseButtons() != Qt::NoButton)
+        return rootItem;
+
+    for (auto it = negativeZStart; it != paintOrderItems.crend(); ++it) {
+        QQuickItem *item = clickableItemAtPosition(*it, rootItem->mapToItem(*it, position));
+        if (item)
+            return item;
+    }
+
+    return nullptr;
+}
+
+QQuickItem *QWaylandQuickOutput::pickClickableItem(const QPointF &position)
+{
+    QQuickWindow *quickWindow = qobject_cast<QQuickWindow *>(window());
+    if (!quickWindow)
+        return nullptr;
+
+    return clickableItemAtPosition(quickWindow->contentItem(), position);
+}
+
 /*!
  * \internal
  */
