@@ -1,35 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2014-2015 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2017-2015 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
+** Copyright (C) 2017 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -38,9 +41,13 @@
 #include "qwaylandsurface.h"
 #include "qwaylandsurface_p.h"
 
+#include "wayland_wrapper/qwlbuffermanager_p.h"
+#include "wayland_wrapper/qwlregion_p.h"
+#include <QtWaylandCompositor/private/qtwaylandcompositorglobal_p.h>
+#if QT_CONFIG(wayland_datadevice)
 #include "wayland_wrapper/qwldatadevice_p.h"
 #include "wayland_wrapper/qwldatadevicemanager_p.h"
-#include "wayland_wrapper/qwlregion_p.h"
+#endif
 
 #include "extensions/qwlextendedsurface_p.h"
 #include "qwaylandinputmethodcontrol_p.h"
@@ -52,7 +59,7 @@
 
 #include <QtWaylandCompositor/private/qwaylandcompositor_p.h>
 #include <QtWaylandCompositor/private/qwaylandview_p.h>
-#include <QtWaylandCompositor/private/qwaylandinput_p.h>
+#include <QtWaylandCompositor/private/qwaylandseat_p.h>
 
 #include <QtCore/private/qobject_p.h>
 
@@ -123,19 +130,20 @@ QWaylandSurfacePrivate::QWaylandSurfacePrivate()
     , compositor(Q_NULLPTR)
     , refCount(1)
     , client(Q_NULLPTR)
-    , buffer(0)
     , role(0)
     , inputRegion(infiniteRegion())
     , bufferScale(1)
     , isCursorSurface(false)
     , destroyed(false)
-    , mapped(false)
+    , hasContent(false)
     , isInitialized(false)
     , contentOrientation(Qt::PrimaryOrientation)
+#if QT_CONFIG(im)
     , inputMethodControl(Q_NULLPTR)
+#endif
     , subsurface(0)
 {
-    pending.buffer = 0;
+    pending.buffer = QWaylandBufferRef();
     pending.newlyAttached = false;
     pending.inputRegion = infiniteRegion();
     pending.bufferScale = 1;
@@ -153,9 +161,6 @@ QWaylandSurfacePrivate::~QWaylandSurfacePrivate()
 
     bufferRef = QWaylandBufferRef();
 
-    for (int i = 0; i < bufferPool.size(); i++)
-        bufferPool[i]->setDestroyIfUnused(true);
-
     foreach (QtWayland::FrameCallback *c, pendingFrameCallbacks)
         c->destroy();
     foreach (QtWayland::FrameCallback *c, frameCallbacks)
@@ -193,9 +198,9 @@ void QWaylandSurfacePrivate::notifyViewsAboutDestruction()
     foreach (QWaylandView *view, views) {
         QWaylandViewPrivate::get(view)->markSurfaceAsDestroyed(q);
     }
-    if (mapped) {
-        mapped = false;
-        emit q->mappedChanged();
+    if (hasContent) {
+        hasContent = false;
+        emit q->hasContentChanged();
     }
 }
 
@@ -237,9 +242,7 @@ void QWaylandSurfacePrivate::surface_destroy(Resource *resource)
 
 void QWaylandSurfacePrivate::surface_attach(Resource *, struct wl_resource *buffer, int x, int y)
 {
-    if (pending.buffer)
-        pending.buffer->disown();
-    pending.buffer = createSurfaceBuffer(buffer);
+    pending.buffer = QWaylandBufferRef(getBuffer(buffer));
     pending.offset = QPoint(x, y);
     pending.newlyAttached = true;
 }
@@ -274,19 +277,37 @@ void QWaylandSurfacePrivate::surface_commit(Resource *)
 {
     Q_Q(QWaylandSurface);
 
-    if (pending.buffer || pending.newlyAttached) {
-        setBackBuffer(pending.buffer, pending.damage);
+    if (pending.buffer.hasBuffer() || pending.newlyAttached)
+        bufferRef = pending.buffer;
+
+    auto buffer = bufferRef.buffer();
+    if (buffer)
+        buffer->setCommitted(pending.damage);
+
+    setSize(bufferRef.size());
+    damage = pending.damage.intersected(QRect(QPoint(), size));
+
+    for (int i = 0; i < views.size(); i++) {
+        views.at(i)->bufferCommitted(bufferRef, damage);
     }
 
-    pending.buffer = 0;
-    pending.offset = QPoint();
-    pending.newlyAttached = false;
-    pending.damage = QRegion();
+    emit q->damaged(damage);
+
+    bool oldHasContent = hasContent;
+    hasContent = bufferRef.hasContent();
+    if (oldHasContent != hasContent)
+        emit q->hasContentChanged();
+
+    if (!pending.offset.isNull())
+        emit q->offsetForNextFrame(pending.offset);
 
     setBufferScale(pending.bufferScale);
 
-    if (buffer)
-        buffer->setCommitted();
+
+    pending.buffer = QWaylandBufferRef();
+    pending.offset = QPoint();
+    pending.newlyAttached = false;
+    pending.damage = QRegion();
 
     frameCallbacks << pendingFrameCallbacks;
     pendingFrameCallbacks.clear();
@@ -326,58 +347,16 @@ void QWaylandSurfacePrivate::surface_set_buffer_scale(QtWaylandServer::wl_surfac
     pending.bufferScale = scale;
 }
 
-void QWaylandSurfacePrivate::setBackBuffer(QtWayland::SurfaceBuffer *b, const QRegion &d)
+QtWayland::ClientBuffer *QWaylandSurfacePrivate::getBuffer(struct ::wl_resource *buffer)
 {
-    Q_Q(QWaylandSurface);
-    buffer = b;
-
-    bufferRef = QWaylandBufferRef(buffer);
-
-    setSize(bufferRef.size());
-    damage = d.intersected(QRect(QPoint(), size));
-
-    for (int i = 0; i < views.size(); i++) {
-        views.at(i)->attach(bufferRef, damage);
-    }
-
-    emit q->damaged(damage);
-
-    bool oldMapped = mapped;
-    mapped = QtWayland::SurfaceBuffer::hasContent(buffer);
-    if (oldMapped != mapped)
-        emit q->mappedChanged();
-
-    if (!pending.offset.isNull())
-        emit q->offsetForNextFrame(pending.offset);
-}
-
-QtWayland::SurfaceBuffer *QWaylandSurfacePrivate::createSurfaceBuffer(struct ::wl_resource *buffer)
-{
-    Q_Q(QWaylandSurface);
-    QtWayland::SurfaceBuffer *newBuffer = 0;
-    for (int i = 0; i < bufferPool.size(); i++) {
-        if (!bufferPool[i]->isRegisteredWithBuffer()) {
-            newBuffer = bufferPool[i];
-            newBuffer->initialize(buffer);
-            break;
-        }
-    }
-
-    if (!newBuffer) {
-        newBuffer = new QtWayland::SurfaceBuffer(q);
-        newBuffer->initialize(buffer);
-        bufferPool.append(newBuffer);
-        if (bufferPool.size() > 3)
-            qWarning() << "Increased buffer pool size to" << bufferPool.size() << "for surface" << q;
-    }
-
-    return newBuffer;
+    QtWayland::BufferManager *bufMan = QWaylandCompositorPrivate::get(compositor)->bufferManager();
+    return bufMan->getBuffer(buffer);
 }
 
 /*!
  * \qmltype WaylandSurface
  * \inqmlmodule QtWayland.Compositor
- * \preliminary
+ * \since 5.8
  * \brief Represents a rectangular area on an output device.
  *
  * This type encapsulates a rectangular area of pixels that is displayed on an output device. It
@@ -387,7 +366,7 @@ QtWayland::SurfaceBuffer *QWaylandSurfacePrivate::createSurfaceBuffer(struct ::w
 /*!
  * \class QWaylandSurface
  * \inmodule QtWaylandCompositor
- * \preliminary
+ * \since 5.8
  * \brief The QWaylandSurface class represents a rectangular area on an output device.
  *
  * This class encapsulates a rectangular area of pixels that is displayed on an output device. It
@@ -431,7 +410,7 @@ QWaylandSurface::~QWaylandSurface()
 }
 
 /*!
- * \qmlmethod void QtWaylandCompositor::WaylandSurface::initialize(object compositor, object client, int id, int version)
+ * \qmlmethod void QtWaylandCompositor::WaylandSurface::initialize(WaylandCompositor compositor, WaylandClient client, int id, int version)
  *
  * Initializes the WaylandSurface with the given \a compositor and \a client, and with the given \a id
  * and \a version.
@@ -448,7 +427,9 @@ void QWaylandSurface::initialize(QWaylandCompositor *compositor, QWaylandClient
     d->client = client;
     d->init(client->client(), id, version);
     d->isInitialized = true;
+#if QT_CONFIG(im)
     d->inputMethodControl = new QWaylandInputMethodControl(this);
+#endif
 #ifndef QT_NO_DEBUG
     QWaylandSurfacePrivate::removeUninitializedSurface(d);
 #endif
@@ -464,7 +445,7 @@ bool QWaylandSurface::isInitialized() const
 }
 
 /*!
- * \qmlproperty object QtWaylandCompositor::WaylandSurface::client
+ * \qmlproperty WaylandClient QtWaylandCompositor::WaylandSurface::client
  *
  * This property holds the client using this WaylandSurface.
  */
@@ -484,20 +465,20 @@ QWaylandClient *QWaylandSurface::client() const
 }
 
 /*!
- * \qmlproperty bool QtWaylandCompositor::WaylandSurface::isMapped
+ * \qmlproperty bool QtWaylandCompositor::WaylandSurface::hasContent
  *
  * This property holds whether the WaylandSurface has content.
  */
 
 /*!
- * \property QWaylandSurface::isMapped
+ * \property QWaylandSurface::hasContent
  *
  * This property holds whether the QWaylandSurface has content.
  */
-bool QWaylandSurface::isMapped() const
+bool QWaylandSurface::hasContent() const
 {
     Q_D(const QWaylandSurface);
-    return d->mapped;
+    return d->hasContent;
 }
 
 /*!
@@ -590,7 +571,7 @@ Qt::ScreenOrientation QWaylandSurface::contentOrientation() const
 QWaylandSurface::Origin QWaylandSurface::origin() const
 {
     Q_D(const QWaylandSurface);
-    return d->buffer ? d->buffer->origin() : QWaylandSurface::OriginTopLeft;
+    return d->bufferRef.origin();
 }
 
 /*!
@@ -685,7 +666,11 @@ bool QWaylandSurface::isDestroyed() const
 void QWaylandSurface::markAsCursorSurface(bool cursorSurface)
 {
     Q_D(QWaylandSurface);
+    if (d->isCursorSurface == cursorSurface)
+        return;
+
     d->isCursorSurface = cursorSurface;
+    emit cursorSurfaceChanged();
 }
 
 bool QWaylandSurface::isCursorSurface() const
@@ -694,36 +679,40 @@ bool QWaylandSurface::isCursorSurface() const
     return d->isCursorSurface;
 }
 
+#if QT_CONFIG(im)
 QWaylandInputMethodControl *QWaylandSurface::inputMethodControl() const
 {
     Q_D(const QWaylandSurface);
     return d->inputMethodControl;
 }
+#endif
 
 /*!
  * Updates the surface with the compositor's retained clipboard selection. Although
  * this is done automatically when the surface receives keyboard focus, this
  * function is useful for updating clients which do not have keyboard focus.
  */
+#if QT_CONFIG(clipboard)
 void QWaylandSurface::updateSelection()
 {
     Q_D(QWaylandSurface);
-    QWaylandInputDevice *inputDevice = d->compositor->defaultInputDevice();
-    if (inputDevice) {
-        const QtWayland::DataDevice *dataDevice = QWaylandInputDevicePrivate::get(inputDevice)->dataDevice();
+    QWaylandSeat *seat = d->compositor->defaultSeat();
+    if (seat) {
+        const QtWayland::DataDevice *dataDevice = QWaylandSeatPrivate::get(seat)->dataDevice();
         if (dataDevice) {
             QWaylandCompositorPrivate::get(d->compositor)->dataDeviceManager()->offerRetainedSelection(
                         dataDevice->resourceMap().value(d->resource()->client())->handle);
         }
     }
 }
+#endif
 
 /*!
- * Returns this QWaylandSurface's throttling view.
+ * Returns this QWaylandSurface's primary view.
  *
- * \sa QWaylandView::advance()
+ * \sa QWaylandView::advance(), QWaylandSurface::setPrimaryView()
  */
-QWaylandView *QWaylandSurface::throttlingView() const
+QWaylandView *QWaylandSurface::primaryView() const
 {
     Q_D(const QWaylandSurface);
     if (d->views.isEmpty())
@@ -732,15 +721,20 @@ QWaylandView *QWaylandSurface::throttlingView() const
 }
 
 /*!
- * Sets this QWaylandSurface's throttling view to \a view, in case there are
- * multiple views of this surface. The throttling view is the view that
+ * Sets this QWaylandSurface's primary view to \a view, in case there are
+ * multiple views of this surface. The primary view is the view that
  * governs the client's refresh rate. It takes care of discarding buffer
  * references when QWaylandView::advance() is called. See the documentation
  * for QWaylandView::advance() for more details.
  *
+ * In shell surface integrations, such as QWaylandWlShellIntegration and
+ * QWaylandXdgShellV5Integration, maximize and fullscreen requests from the
+ * client will only have an effect if the integration has the primary view
+ * of the surface.
+ *
  * \sa QWaylandView::advance()
  */
-void QWaylandSurface::setThrottlingView(QWaylandView *view)
+void QWaylandSurface::setPrimaryView(QWaylandView *view)
 {
     Q_D(QWaylandSurface);
 
@@ -835,8 +829,7 @@ void QWaylandSurfacePrivate::refView(QWaylandView *view)
 
     views.append(view);
     ref();
-    QWaylandBufferRef ref(buffer);
-    view->attach(ref, QRect(QPoint(0,0), ref.size()));
+    view->bufferCommitted(bufferRef, QRect(QPoint(0,0), bufferRef.size()));
 }
 
 void QWaylandSurfacePrivate::derefView(QWaylandView *view)
@@ -894,4 +887,16 @@ void QWaylandSurfacePrivate::Subsurface::subsurface_set_desync(wl_subsurface::Re
     qDebug() << Q_FUNC_INFO;
 }
 
+/*!
+ * \qmlsignal void QtWaylandCompositor::WaylandSurface::dragStarted(WaylandDrag drag)
+ *
+ * This signal is emitted when a \a drag has started from this surface.
+ */
+
+/*!
+ * \fn void QWaylandSurface::dragStarted(QWaylandDrag *drag)
+ *
+ * This signal is emitted when a \a drag has started from this surface.
+ */
+
 QT_END_NAMESPACE
