@@ -1,34 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2017 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,11 +44,13 @@
 #include "qwaylandsurface_p.h"
 #include "qwldatadevicemanager_p.h"
 
+#if QT_CONFIG(draganddrop)
 #include "qwaylanddrag.h"
+#endif
 #include "qwaylandview.h"
 #include <QtWaylandCompositor/QWaylandClient>
 #include <QtWaylandCompositor/private/qwaylandcompositor_p.h>
-#include <QtWaylandCompositor/private/qwaylandinput_p.h>
+#include <QtWaylandCompositor/private/qwaylandseat_p.h>
 #include <QtWaylandCompositor/private/qwaylandpointer_p.h>
 
 #include <QtCore/QPointF>
@@ -55,16 +60,19 @@ QT_BEGIN_NAMESPACE
 
 namespace QtWayland {
 
-DataDevice::DataDevice(QWaylandInputDevice *inputDevice)
+DataDevice::DataDevice(QWaylandSeat *seat)
     : wl_data_device()
-    , m_compositor(inputDevice->compositor())
-    , m_inputDevice(inputDevice)
+    , m_compositor(seat->compositor())
+    , m_seat(seat)
     , m_selectionSource(0)
+#if QT_CONFIG(draganddrop)
     , m_dragClient(0)
     , m_dragDataSource(0)
     , m_dragFocus(0)
     , m_dragFocusResource(0)
     , m_dragIcon(0)
+    , m_dragOrigin(nullptr)
+#endif
 {
 }
 
@@ -84,6 +92,13 @@ void DataDevice::setFocus(QWaylandClient *focusClient)
     }
 }
 
+void DataDevice::sourceDestroyed(DataSource *source)
+{
+    if (m_selectionSource == source)
+        m_selectionSource = 0;
+}
+
+#if QT_CONFIG(draganddrop)
 void DataDevice::setDragFocus(QWaylandSurface *focus, const QPointF &localPosition)
 {
     if (m_dragFocusResource) {
@@ -123,10 +138,9 @@ QWaylandSurface *DataDevice::dragIcon() const
     return m_dragIcon;
 }
 
-void DataDevice::sourceDestroyed(DataSource *source)
+QWaylandSurface *DataDevice::dragOrigin() const
 {
-    if (m_selectionSource == source)
-        m_selectionSource = 0;
+    return m_dragOrigin;
 }
 
 void DataDevice::dragMove(QWaylandSurface *target, const QPointF &pos)
@@ -148,6 +162,7 @@ void DataDevice::drop()
     } else {
         m_dragDataSource->cancel();
     }
+    m_dragOrigin = nullptr;
     setDragIcon(nullptr);
 }
 
@@ -160,14 +175,25 @@ void DataDevice::data_device_start_drag(Resource *resource, struct ::wl_resource
 {
     m_dragClient = resource->client();
     m_dragDataSource = source ? DataSource::fromResource(source) : 0;
+    m_dragOrigin = QWaylandSurface::fromResource(origin);
+    QWaylandDrag *drag = m_seat->drag();
     setDragIcon(icon ? QWaylandSurface::fromResource(icon) : nullptr);
-    Q_EMIT m_inputDevice->drag()->dragStarted();
+    Q_EMIT drag->dragStarted();
+    Q_EMIT m_dragOrigin->dragStarted(drag);
 
     Q_UNUSED(serial);
-    Q_UNUSED(origin);
     //### need to verify that we have an implicit grab with this serial
 }
 
+void DataDevice::setDragIcon(QWaylandSurface *icon)
+{
+    if (icon == m_dragIcon)
+        return;
+    m_dragIcon = icon;
+    Q_EMIT m_seat->drag()->iconChanged();
+}
+#endif // QT_CONFIG(draganddrop)
+
 void DataDevice::data_device_set_selection(Resource *, struct ::wl_resource *source, uint32_t serial)
 {
     Q_UNUSED(serial);
@@ -182,7 +208,7 @@ void DataDevice::data_device_set_selection(Resource *, struct ::wl_resource *sou
     if (m_selectionSource)
         m_selectionSource->setDevice(this);
 
-    QWaylandClient *focusClient = m_inputDevice->keyboard()->focusClient();
+    QWaylandClient *focusClient = m_seat->keyboard()->focusClient();
     Resource *resource = focusClient ? resourceMap().value(focusClient->client()) : 0;
 
     if (resource && m_selectionSource) {
@@ -193,13 +219,6 @@ void DataDevice::data_device_set_selection(Resource *, struct ::wl_resource *sou
     }
 }
 
-void DataDevice::setDragIcon(QWaylandSurface *icon)
-{
-    if (icon == m_dragIcon)
-        return;
-    m_dragIcon = icon;
-    Q_EMIT m_inputDevice->drag()->iconChanged();
-}
 
 }
 
