@@ -1,34 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2017 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -40,6 +43,8 @@
 #include <qpa/qplatformnativeinterface.h>
 #include <QtGui/QGuiApplication>
 #include <QtGui/QOpenGLContext>
+#include <QtGui/QOpenGLTexture>
+#include <QtGui/QOffscreenSurface>
 #include <qpa/qplatformscreen.h>
 #include <QtGui/QWindow>
 #include <QtCore/QPointer>
@@ -48,7 +53,7 @@
 #include <QMutex>
 #include <QMutexLocker>
 #include <QtCore/private/qcore_unix_p.h>
-#include <QtPlatformSupport/private/qeglstreamconvenience_p.h>
+#include <QtEglSupport/private/qeglstreamconvenience_p.h>
 
 #ifndef GL_TEXTURE_EXTERNAL_OES
 #define GL_TEXTURE_EXTERNAL_OES     0x8D65
@@ -58,6 +63,10 @@
 #define EGL_WAYLAND_BUFFER_WL       0x31D5
 #endif
 
+#ifndef EGL_WAYLAND_EGLSTREAM_WL
+#define EGL_WAYLAND_EGLSTREAM_WL    0x334B
+#endif
+
 #ifndef EGL_WAYLAND_PLANE_WL
 #define EGL_WAYLAND_PLANE_WL        0x31D6
 #endif
@@ -90,6 +99,10 @@
 #define EGL_TEXTURE_Y_XUXV_WL       0x31D9
 #endif
 
+#ifndef EGL_PLATFORM_X11_KHR
+#define EGL_PLATFORM_X11_KHR        0x31D5
+#endif
+
 /* Needed for compatibility with Mesa older than 10.0. */
 typedef EGLBoolean (EGLAPIENTRYP PFNEGLQUERYWAYLANDBUFFERWL_compat) (EGLDisplay dpy, struct wl_resource *buffer, EGLint attribute, EGLint *value);
 
@@ -110,17 +123,50 @@ typedef void (GL_APIENTRYP PFNGLEGLIMAGETARGETRENDERBUFFERSTORAGEOESPROC) (GLenu
 
 QT_BEGIN_NAMESPACE
 
+static const char *
+egl_error_string(EGLint code)
+{
+#define MYERRCODE(x) case x: return #x;
+    switch (code) {
+    MYERRCODE(EGL_SUCCESS)
+    MYERRCODE(EGL_NOT_INITIALIZED)
+    MYERRCODE(EGL_BAD_ACCESS)
+    MYERRCODE(EGL_BAD_ALLOC)
+    MYERRCODE(EGL_BAD_ATTRIBUTE)
+    MYERRCODE(EGL_BAD_CONTEXT)
+    MYERRCODE(EGL_BAD_CONFIG)
+    MYERRCODE(EGL_BAD_CURRENT_SURFACE)
+    MYERRCODE(EGL_BAD_DISPLAY)
+    MYERRCODE(EGL_BAD_SURFACE)
+    MYERRCODE(EGL_BAD_MATCH)
+    MYERRCODE(EGL_BAD_PARAMETER)
+    MYERRCODE(EGL_BAD_NATIVE_PIXMAP)
+    MYERRCODE(EGL_BAD_NATIVE_WINDOW)
+    MYERRCODE(EGL_CONTEXT_LOST)
+    default:
+        return "unknown";
+    }
+#undef MYERRCODE
+}
+
 struct BufferState
 {
     BufferState();
 
+    enum EglMode {
+        ModeUninitialized,
+        ModeEGLImage,
+        ModeEGLStream
+    };
+
     EGLint egl_format;
     QVarLengthArray<EGLImageKHR, 3> egl_images;
+    QOpenGLTexture *textures[3] = {};
     EGLStreamKHR egl_stream;
-    GLuint eglstream_texture;
 
     bool isYInverted;
     QSize size;
+    EglMode eglMode;
 };
 
 class WaylandEglClientBufferIntegrationPrivate
@@ -128,18 +174,18 @@ class WaylandEglClientBufferIntegrationPrivate
 public:
     WaylandEglClientBufferIntegrationPrivate();
 
-    void attach(struct ::wl_resource *buffer);
-    void attach_egl_texture(struct ::wl_resource *buffer, EGLint format);
-    void attach_egl_fd_texture(struct ::wl_resource *buffer, EGLNativeFileDescriptorKHR streamFd);
-    void register_buffer(struct ::wl_resource *buffer, BufferState state);
-    void bindBuffer(struct ::wl_resource *buffer);
-
-    static void handle_buffer_destroy(wl_listener *listener, void *data);
+    void initBuffer(WaylandEglClientBuffer *buffer);
+    void initEglTexture(WaylandEglClientBuffer *buffer, EGLint format);
+    bool ensureContext();
+    bool initEglStream(WaylandEglClientBuffer *buffer, struct ::wl_resource *bufferHandle);
+    void handleEglstreamTexture(WaylandEglClientBuffer *buffer, wl_resource *bufferHandle);
+    void registerBuffer(struct ::wl_resource *buffer, BufferState state);
 
     EGLDisplay egl_display;
     bool valid;
     bool display_bound;
-    QHash<struct ::wl_resource *, BufferState> buffers;
+    QOffscreenSurface *offscreenSurface = nullptr;
+    QOpenGLContext *localContext = nullptr;
 
     PFNEGLBINDWAYLANDDISPLAYWL egl_bind_wayland_display;
     PFNEGLUNBINDWAYLANDDISPLAYWL egl_unbind_wayland_display;
@@ -151,25 +197,20 @@ public:
     PFNGLEGLIMAGETARGETTEXTURE2DOESPROC gl_egl_image_target_texture_2d;
 
     QEGLStreamConvenience *funcs;
-};
-
-struct buffer_destroy_listener : wl_listener
-{
-    buffer_destroy_listener()
-        : d(0)
-    {
-        notify = WaylandEglClientBufferIntegrationPrivate::handle_buffer_destroy;
-        wl_list_init(&this->link);
+    static WaylandEglClientBufferIntegrationPrivate *get(WaylandEglClientBufferIntegration *integration) {
+        return shuttingDown ? nullptr : integration->d_ptr.data();
     }
 
-    WaylandEglClientBufferIntegrationPrivate *d;
+    static bool shuttingDown;
 };
 
+bool WaylandEglClientBufferIntegrationPrivate::shuttingDown = false;
+
 BufferState::BufferState()
     : egl_format(EGL_TEXTURE_RGBA)
     , egl_stream(EGL_NO_STREAM_KHR)
-    , eglstream_texture(0)
     , isYInverted(true)
+    , eglMode(ModeUninitialized)
 {}
 
 WaylandEglClientBufferIntegrationPrivate::WaylandEglClientBufferIntegrationPrivate()
@@ -183,21 +224,21 @@ WaylandEglClientBufferIntegrationPrivate::WaylandEglClientBufferIntegrationPriva
     , egl_destroy_image(0)
     , gl_egl_image_target_texture_2d(0)
     , funcs(Q_NULLPTR)
-{}
-
-void WaylandEglClientBufferIntegrationPrivate::attach(struct ::wl_resource *buffer)
 {
-    EGLint format;
-    EGLNativeFileDescriptorKHR streamFd = EGL_NO_FILE_DESCRIPTOR_KHR;
-
-    if (egl_query_wayland_buffer(egl_display, buffer, EGL_TEXTURE_FORMAT, &format))
-        attach_egl_texture(buffer, format);
-    else if (egl_query_wayland_buffer(egl_display, buffer, EGL_WAYLAND_BUFFER_WL, &streamFd))
-        attach_egl_fd_texture(buffer, streamFd);
 }
 
-void WaylandEglClientBufferIntegrationPrivate::attach_egl_texture(struct ::wl_resource *buffer, EGLint format)
+void WaylandEglClientBufferIntegrationPrivate::initBuffer(WaylandEglClientBuffer *buffer)
 {
+    EGLint format;
+
+    if (egl_query_wayland_buffer(egl_display, buffer->waylandBufferHandle(), EGL_TEXTURE_FORMAT, &format))
+        initEglTexture(buffer, format);
+}
+
+void WaylandEglClientBufferIntegrationPrivate::initEglTexture(WaylandEglClientBuffer *buffer, EGLint format)
+{
+// Non-streaming case
+
     // Resolving GL functions may need a context current, so do it only here.
     if (!gl_egl_image_target_texture_2d)
         gl_egl_image_target_texture_2d = reinterpret_cast<PFNGLEGLIMAGETARGETTEXTURE2DOESPROC>(eglGetProcAddress("glEGLImageTargetTexture2DOES"));
@@ -207,12 +248,13 @@ void WaylandEglClientBufferIntegrationPrivate::attach_egl_texture(struct ::wl_re
         return;
     }
 
-    BufferState state;
+    BufferState &state = *buffer->d;
     state.egl_format = format;
+    state.eglMode = BufferState::ModeEGLImage;
 
 #if defined(EGL_WAYLAND_Y_INVERTED_WL)
     EGLint isYInverted;
-    EGLBoolean ret = egl_query_wayland_buffer(egl_display, buffer, EGL_WAYLAND_Y_INVERTED_WL, &isYInverted);
+    EGLBoolean ret = egl_query_wayland_buffer(egl_display, buffer->waylandBufferHandle(), EGL_WAYLAND_Y_INVERTED_WL, &isYInverted);
     // Yes, this looks strange, but the specification says that EGL_FALSE return
     // value (not supported) should be treated the same as EGL_TRUE return value
     // and EGL_TRUE in value.
@@ -244,130 +286,111 @@ void WaylandEglClientBufferIntegrationPrivate::attach_egl_texture(struct ::wl_re
         EGLImageKHR image = egl_create_image(egl_display,
                                              EGL_NO_CONTEXT,
                                              EGL_WAYLAND_BUFFER_WL,
-                                             buffer,
+                                             buffer->waylandBufferHandle(),
                                              attribs);
 
         if (image == EGL_NO_IMAGE_KHR)
             qWarning("failed to create EGL image for plane %d", i);
 
         state.egl_images << image;
+        state.textures[i] = nullptr;
     }
-
-    register_buffer(buffer, state);
 }
 
-void WaylandEglClientBufferIntegrationPrivate::attach_egl_fd_texture(struct ::wl_resource *buffer, EGLNativeFileDescriptorKHR streamFd)
+bool WaylandEglClientBufferIntegrationPrivate::ensureContext()
 {
-    BufferState state;
+    bool localContextNeeded = false;
+    if (!QOpenGLContext::currentContext()) {
+        if (!localContext && QOpenGLContext::globalShareContext()) {
+            localContext = new QOpenGLContext;
+            localContext->setShareContext(QOpenGLContext::globalShareContext());
+            localContext->create();
+        }
+        if (localContext) {
+            if (!offscreenSurface) {
+                offscreenSurface = new QOffscreenSurface;
+                offscreenSurface->setFormat(localContext->format());
+                offscreenSurface->create();
+            }
+            localContext->makeCurrent(offscreenSurface);
+            localContextNeeded = true;
+        }
+    }
+    return localContextNeeded;
+}
 
+bool WaylandEglClientBufferIntegrationPrivate::initEglStream(WaylandEglClientBuffer *buffer, wl_resource *bufferHandle)
+{
+    BufferState &state = *buffer->d;
     state.egl_format = EGL_TEXTURE_EXTERNAL_WL;
     state.isYInverted = false;
-    state.egl_stream = funcs->create_stream_from_file_descriptor(egl_display, streamFd);
 
-    close(streamFd);
+    EGLNativeFileDescriptorKHR streamFd = EGL_NO_FILE_DESCRIPTOR_KHR;
+
+    if (egl_query_wayland_buffer(egl_display, bufferHandle, EGL_WAYLAND_BUFFER_WL, &streamFd)) {
+        state.egl_stream = funcs->create_stream_from_file_descriptor(egl_display, streamFd);
+        close(streamFd);
+    } else {
+        EGLAttrib stream_attribs[] = {
+            EGL_WAYLAND_EGLSTREAM_WL, (EGLAttrib)bufferHandle,
+            EGL_NONE
+        };
+        state.egl_stream = funcs->create_stream_attrib_nv(egl_display, stream_attribs);
+    }
 
     if (state.egl_stream == EGL_NO_STREAM_KHR) {
         qWarning("%s:%d: eglCreateStreamFromFileDescriptorKHR failed: 0x%x", Q_FUNC_INFO, __LINE__, eglGetError());
-        return;
+        return false;
     }
+    state.eglMode = BufferState::ModeEGLStream;
 
-    if (!QOpenGLContext::currentContext())
+    if (!QOpenGLContext::currentContext()) {
         qWarning("EglClientBufferIntegration: creating texture with no current context");
-
-    //TODO This texture might end up in a different context than the quick item which wants to use it, this needs to be fixed somehow.
-    glGenTextures(1, &state.eglstream_texture);
-    glActiveTexture(GL_TEXTURE0);
-    glBindTexture(GL_TEXTURE_EXTERNAL_OES, state.eglstream_texture);
-    glTexParameteri(GL_TEXTURE_EXTERNAL_OES, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_EXTERNAL_OES, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_EXTERNAL_OES, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-    glTexParameteri(GL_TEXTURE_EXTERNAL_OES, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-
-    register_buffer(buffer, state);
-
-    bindBuffer(buffer);
-}
-
-void WaylandEglClientBufferIntegrationPrivate::register_buffer(struct ::wl_resource *buffer, BufferState state)
-{
-    Q_ASSERT(!buffers.contains(buffer));
-
-    EGLint width, height;
-    egl_query_wayland_buffer(egl_display, buffer, EGL_WIDTH, &width);
-    egl_query_wayland_buffer(egl_display, buffer, EGL_HEIGHT, &height);
-    state.size = QSize(width, height);
-
-    buffers[buffer] = state;
-
-    buffer_destroy_listener *destroy_listener = new buffer_destroy_listener;
-    destroy_listener->d = this;
-    wl_signal_add(&buffer->destroy_signal, destroy_listener);
-}
-
-void WaylandEglClientBufferIntegrationPrivate::bindBuffer(struct ::wl_resource *buffer)
-{
-    if (!valid) {
-        qWarning("QtCompositor: bindTextureToBuffer() failed");
-        return;
+        return false;
     }
 
-    if (!buffer || !buffers.contains(buffer))
-        return;
+    auto texture = new QOpenGLTexture(static_cast<QOpenGLTexture::Target>(GL_TEXTURE_EXTERNAL_OES));
+    texture->create();
+    state.textures[0] = texture; // TODO: support multiple planes for the streaming case
 
-    const BufferState state = buffers.value(buffer);
+    texture->bind();
 
-    if (state.egl_stream != EGL_NO_STREAM_KHR) {
-        EGLint stream_state;
-        funcs->query_stream(egl_display, state.egl_stream, EGL_STREAM_STATE_KHR, &stream_state);
-
-        if (stream_state == EGL_STREAM_STATE_CREATED_KHR)
-            if (funcs->stream_consumer_gltexture(egl_display, state.egl_stream) != EGL_TRUE)
-                qWarning("%s:%d: eglStreamConsumerGLTextureExternalKHR failed: 0x%x", Q_FUNC_INFO, __LINE__, eglGetError());
-    } else {
-        GLint previousTexture = GL_TEXTURE0;
-        glGetIntegerv(GL_ACTIVE_TEXTURE, &previousTexture);
-
-        const GLenum target = (state.egl_format == EGL_TEXTURE_EXTERNAL_WL) ? GL_TEXTURE_EXTERNAL_OES
-                                                                            : GL_TEXTURE_2D;
-
-        for (int i = 0; i < state.egl_images.size(); i++) {
-            glActiveTexture(GL_TEXTURE0 + i);
-            glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-            glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-            glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-            glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-            gl_egl_image_target_texture_2d(target, state.egl_images[i]);
-        }
-
-        glActiveTexture(previousTexture);
+    auto newStream = funcs->stream_consumer_gltexture(egl_display, state.egl_stream);
+    if (!newStream) {
+        EGLint code = eglGetError();
+        qWarning() << "Could not initialize EGLStream:" << egl_error_string(code) << hex << (long)code;
+        funcs->destroy_stream(egl_display, state.egl_stream);
+        state.egl_stream = EGL_NO_STREAM_KHR;
+        return false;
     }
+    return true;
 }
 
-void WaylandEglClientBufferIntegrationPrivate::handle_buffer_destroy(wl_listener *listener, void *data)
+void WaylandEglClientBufferIntegrationPrivate::handleEglstreamTexture(WaylandEglClientBuffer *buffer, struct ::wl_resource *bufferHandle)
 {
-    buffer_destroy_listener *destroy_listener = static_cast<buffer_destroy_listener *>(listener);
-    WaylandEglClientBufferIntegrationPrivate *self = destroy_listener->d;
-    struct ::wl_resource *buffer = static_cast<struct ::wl_resource *>(data);
+    bool usingLocalContext = ensureContext();
 
-    wl_list_remove(&destroy_listener->link);
-    delete destroy_listener;
+    if (buffer->d->eglMode == BufferState::ModeUninitialized) {
+        bool streamOK = initEglStream(buffer, bufferHandle);
+        if (!streamOK)
+            return;
+    }
 
-    if (!self->buffers.contains(buffer))
-        return;
+    BufferState &state = *buffer->d;
+    auto texture = state.textures[0];
 
-    Q_ASSERT(self);
-    Q_ASSERT(buffer);
+    // EGLStream requires calling acquire on every frame.
+    texture->bind();
+    EGLint stream_state;
+    funcs->query_stream(egl_display, state.egl_stream, EGL_STREAM_STATE_KHR, &stream_state);
 
-    BufferState state = self->buffers.take(buffer);
+    if (stream_state == EGL_STREAM_STATE_NEW_FRAME_AVAILABLE_KHR) {
+        if (funcs->stream_consumer_acquire(egl_display, state.egl_stream) != EGL_TRUE)
+            qWarning("%s:%d: eglStreamConsumerAcquireKHR failed: 0x%x", Q_FUNC_INFO, __LINE__, eglGetError());
+    }
 
-    // We would need to delete the texture of the egl_stream here, but we can't as this breaks the
-    // texture of the new stream when the window is resized. It seems wayland takes care to delete the texture for us.
-
-    for (int i = 0; i < state.egl_images.size(); i++)
-        self->egl_destroy_image(self->egl_display, state.egl_images[i]);
-
-    if (state.egl_stream != EGL_NO_STREAM_KHR)
-        self->funcs->destroy_stream(self->egl_display, state.egl_stream);
+    if (usingLocalContext)
+        localContext->doneCurrent();
 }
 
 WaylandEglClientBufferIntegration::WaylandEglClientBufferIntegration()
@@ -376,6 +399,11 @@ WaylandEglClientBufferIntegration::WaylandEglClientBufferIntegration()
 {
 }
 
+WaylandEglClientBufferIntegration::~WaylandEglClientBufferIntegration()
+{
+    WaylandEglClientBufferIntegrationPrivate::shuttingDown = true;
+}
+
 void WaylandEglClientBufferIntegration::initializeHardware(struct wl_display *display)
 {
     Q_D(WaylandEglClientBufferIntegration);
@@ -438,17 +466,45 @@ void WaylandEglClientBufferIntegration::initializeHardware(struct wl_display *di
     d->valid = true;
 }
 
-void WaylandEglClientBufferIntegration::initializeBuffer(struct ::wl_resource *buffer)
+QtWayland::ClientBuffer *WaylandEglClientBufferIntegration::createBufferFor(wl_resource *buffer)
 {
-    Q_D(WaylandEglClientBufferIntegration);
-
     if (wl_shm_buffer_get(buffer))
-        return;
+        return nullptr;
+    return new WaylandEglClientBuffer(this, buffer);
+}
 
-    if (!buffer || d->buffers.contains(buffer))
-        return;
+WaylandEglClientBuffer::WaylandEglClientBuffer(WaylandEglClientBufferIntegration *integration, wl_resource *buffer)
+    : ClientBuffer(buffer)
+    , m_integration(integration)
+{
+    auto *p = WaylandEglClientBufferIntegrationPrivate::get(m_integration);
+    d = new BufferState;
+    if (buffer && !wl_shm_buffer_get(buffer)) {
+        EGLint width, height;
+        p->egl_query_wayland_buffer(p->egl_display, buffer, EGL_WIDTH, &width);
+        p->egl_query_wayland_buffer(p->egl_display, buffer, EGL_HEIGHT, &height);
+        d->size = QSize(width, height);
 
-    d->attach(buffer);
+        p->initBuffer(this);
+    }
+}
+
+
+WaylandEglClientBuffer::~WaylandEglClientBuffer()
+{
+    auto *p = WaylandEglClientBufferIntegrationPrivate::get(m_integration);
+
+    if (p) {
+        for (auto image : d->egl_images)
+            p->egl_destroy_image(p->egl_display, image);
+
+        if (d->egl_stream)
+            p->funcs->destroy_stream(p->egl_display, d->egl_stream);
+
+        for (auto *texture : d->textures)
+            delete texture;
+    }
+    delete d;
 }
 
 static QWaylandBufferRef::BufferFormatEgl formatFromEglFormat(EGLint format) {
@@ -470,117 +526,91 @@ static QWaylandBufferRef::BufferFormatEgl formatFromEglFormat(EGLint format) {
     return QWaylandBufferRef::BufferFormatEgl_RGBA;
 }
 
-QWaylandBufferRef::BufferFormatEgl WaylandEglClientBufferIntegration::bufferFormat(wl_resource *buffer)
-{
-    Q_D(const WaylandEglClientBufferIntegration);
-    return formatFromEglFormat(d->buffers.value(buffer).egl_format);
-}
-
-uint WaylandEglClientBufferIntegration::textureForBuffer(wl_resource *buffer, int plane)
-{
-    Q_UNUSED(plane)
-    Q_D(WaylandEglClientBufferIntegration);
-    if (!buffer)
-        return 0;
-
-    const BufferState state = d->buffers.value(buffer);
-    return state.eglstream_texture;
-}
-
-void WaylandEglClientBufferIntegration::bindTextureToBuffer(struct ::wl_resource *buffer)
-{
-    Q_D(WaylandEglClientBufferIntegration);
-    d->bindBuffer(buffer);
-}
-
-// Update is only needed for the EGLStream path as that requires calling acquire
-// on every frame. bindTextureToBuffer() is typically invoked only upon attach
-// so that is insufficient.
-void WaylandEglClientBufferIntegration::updateTextureForBuffer(struct ::wl_resource *buffer)
-{
-    Q_D(WaylandEglClientBufferIntegration);
-    if (!d->valid) {
-        qWarning("QtCompositor: updateTextureForBuffer() failed");
-        return;
-    }
-
-    if (!buffer)
-        return;
-
-    const BufferState state = d->buffers.value(buffer);
-
-    if (state.egl_stream != EGL_NO_STREAM_KHR) {
-        EGLint stream_state;
-        d->funcs->query_stream(d->egl_display, state.egl_stream, EGL_STREAM_STATE_KHR, &stream_state);
-
-        if (stream_state == EGL_STREAM_STATE_NEW_FRAME_AVAILABLE_KHR) {
-            if (d->funcs->stream_consumer_acquire(d->egl_display, state.egl_stream) != EGL_TRUE)
-                qWarning("%s:%d: eglStreamConsumerAcquireKHR failed: 0x%x", Q_FUNC_INFO, __LINE__, eglGetError());
-        }
+static QOpenGLTexture::TextureFormat openGLFormatFromEglFormat(EGLint format) {
+    switch (format) {
+    case EGL_TEXTURE_RGB:
+        return QOpenGLTexture::RGBFormat;
+    case EGL_TEXTURE_RGBA:
+        return QOpenGLTexture::RGBAFormat;
+    default:
+        return QOpenGLTexture::NoFormat;
     }
 }
 
-QWaylandSurface::Origin WaylandEglClientBufferIntegration::origin(struct ::wl_resource *buffer) const
+QWaylandBufferRef::BufferFormatEgl WaylandEglClientBuffer::bufferFormatEgl() const
 {
-    Q_D(const WaylandEglClientBufferIntegration);
-
-    if (d->buffers.contains(buffer))
-        return d->buffers[buffer].isYInverted ? QWaylandSurface::OriginTopLeft : QWaylandSurface::OriginBottomLeft;
-
-#if defined(EGL_WAYLAND_Y_INVERTED_WL)
-    EGLint isYInverted;
-    EGLBoolean ret = EGL_FALSE;
-    if (buffer)
-        ret = d->egl_query_wayland_buffer(d->egl_display, buffer, EGL_WAYLAND_Y_INVERTED_WL, &isYInverted);
-    // Yes, this looks strange, but the specification says that EGL_FALSE return
-    // value (not supported) should be treated the same as EGL_TRUE return value
-    // and EGL_TRUE in value.
-    if (ret == EGL_FALSE || isYInverted == EGL_TRUE)
-        return QWaylandSurface::OriginTopLeft;
-    return QWaylandSurface::OriginBottomLeft;
-#endif
-
-    return QtWayland::ClientBufferIntegration::origin(buffer);
+    return formatFromEglFormat(d->egl_format);
 }
 
-
-void *WaylandEglClientBufferIntegration::lockNativeBuffer(struct ::wl_resource *buffer) const
+QOpenGLTexture *WaylandEglClientBuffer::toOpenGlTexture(int plane)
 {
-    Q_D(const WaylandEglClientBufferIntegration);
+    if (!m_buffer)
+        return nullptr;
 
-    if (d->buffers.contains(buffer) && d->buffers[buffer].egl_stream != EGL_NO_STREAM_KHR)
+    auto texture = d->textures[plane];
+    if (d->eglMode == BufferState::ModeEGLStream)
+        return texture; // EGLStreams texture is maintained by handle_eglstream_texture()
+
+    auto *p = WaylandEglClientBufferIntegrationPrivate::get(m_integration);
+    const auto target = static_cast<QOpenGLTexture::Target>(d->egl_format == EGL_TEXTURE_EXTERNAL_WL ? GL_TEXTURE_EXTERNAL_OES
+                                                                        : GL_TEXTURE_2D);
+    if (!texture) {
+        texture = new QOpenGLTexture(target);
+        texture->setFormat(openGLFormatFromEglFormat(d->egl_format));
+        texture->setSize(d->size.width(), d->size.height());
+        texture->create();
+        d->textures[plane] = texture;
+    }
+
+    if (m_textureDirty) {
+        texture->bind();
+        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+        p->gl_egl_image_target_texture_2d(target, d->egl_images[plane]);
+    }
+    return texture;
+}
+
+void WaylandEglClientBuffer::setCommitted(QRegion &damage)
+{
+    ClientBuffer::setCommitted(damage);
+    if (d->eglMode == BufferState::ModeEGLStream || d->eglMode == BufferState::ModeUninitialized) {
+        auto *p = WaylandEglClientBufferIntegrationPrivate::get(m_integration);
+        p->handleEglstreamTexture(this, waylandBufferHandle());
+    }
+}
+
+QWaylandSurface::Origin WaylandEglClientBuffer::origin() const
+{
+    return d->isYInverted ? QWaylandSurface::OriginTopLeft : QWaylandSurface::OriginBottomLeft;
+}
+
+quintptr WaylandEglClientBuffer::lockNativeBuffer()
+{
+    auto *p = WaylandEglClientBufferIntegrationPrivate::get(m_integration);
+
+    if (d->egl_stream != EGL_NO_STREAM_KHR)
         return 0;
 
-    EGLImageKHR image = d->egl_create_image(d->egl_display, EGL_NO_CONTEXT,
+    EGLImageKHR image = p->egl_create_image(p->egl_display, EGL_NO_CONTEXT,
                                           EGL_WAYLAND_BUFFER_WL,
-                                          buffer, NULL);
-    return image;
+                                          m_buffer, NULL);
+    return reinterpret_cast<quintptr>(image);
 }
 
-void WaylandEglClientBufferIntegration::unlockNativeBuffer(void *native_buffer) const
+void WaylandEglClientBuffer::unlockNativeBuffer(quintptr native_buffer) const
 {
-    Q_D(const WaylandEglClientBufferIntegration);
-
     if (!native_buffer)
         return;
 
-    EGLImageKHR image = static_cast<EGLImageKHR>(native_buffer);
-    d->egl_destroy_image(d->egl_display, image);
+    auto *p = WaylandEglClientBufferIntegrationPrivate::get(m_integration);
+
+    EGLImageKHR image = reinterpret_cast<EGLImageKHR>(native_buffer);
+    p->egl_destroy_image(p->egl_display, image);
 }
 
-QSize WaylandEglClientBufferIntegration::bufferSize(struct ::wl_resource *buffer) const
+QSize WaylandEglClientBuffer::size() const
 {
-    Q_D(const WaylandEglClientBufferIntegration);
-
-    if (d->buffers.contains(buffer)) {
-        return d->buffers[buffer].size;
-    } else {
-        int width, height;
-        d->egl_query_wayland_buffer(d->egl_display, buffer, EGL_WIDTH, &width);
-        d->egl_query_wayland_buffer(d->egl_display, buffer, EGL_HEIGHT, &height);
-
-        return QSize(width, height);
-    }
+    return d->size;
 }
 
 QT_END_NAMESPACE
