@@ -35,8 +35,12 @@
 #include <QMimeData>
 #include <QPixmap>
 #include <QDrag>
+#include <QWindow>
+#include <QOpenGLWindow>
 
 #include <QtTest/QtTest>
+#include <QtWaylandClient/private/qwaylandintegration_p.h>
+#include <QtGui/private/qguiapplication_p.h>
 
 static const QSize screenSize(1600, 1200);
 
@@ -96,6 +100,8 @@ public:
         ++touchEventCount;
     }
 
+    QPoint frameOffset() const { return QPoint(frameMargins().left(), frameMargins().top()); }
+
     int focusInEventCount;
     int focusOutEventCount;
     int keyPressEventCount;
@@ -108,6 +114,25 @@ public:
     QPoint mousePressPos;
 };
 
+class TestGlWindow : public QOpenGLWindow
+{
+    Q_OBJECT
+
+public:
+    TestGlWindow();
+
+protected:
+    void paintGL() override;
+};
+
+TestGlWindow::TestGlWindow()
+{}
+
+void TestGlWindow::paintGL()
+{
+    glClear(GL_COLOR_BUFFER_BIT);
+}
+
 class tst_WaylandClient : public QObject
 {
     Q_OBJECT
@@ -142,6 +167,10 @@ private slots:
     void backingStore();
     void touchDrag();
     void mouseDrag();
+    void dontCrashOnMultipleCommits();
+    void hiddenTransientParent();
+    void hiddenPopupParent();
+    void glWindow();
 
 private:
     MockCompositor *compositor;
@@ -196,24 +225,26 @@ void tst_WaylandClient::events()
     QTRY_COMPARE(window.keyReleaseEventCount, 1);
     QCOMPARE(window.keyCode, keyCode);
 
-    QPoint mousePressPos(16, 16);
-    QCOMPARE(window.mousePressEventCount, 0);
-    compositor->sendMousePress(surface, mousePressPos);
-    QTRY_COMPARE(window.mousePressEventCount, 1);
-    QTRY_COMPARE(window.mousePressPos, mousePressPos);
-
-    QCOMPARE(window.mouseReleaseEventCount, 0);
-    compositor->sendMouseRelease(surface);
-    QTRY_COMPARE(window.mouseReleaseEventCount, 1);
-
     const int touchId = 0;
-    compositor->sendTouchDown(surface, QPoint(10, 10), touchId);
+    compositor->sendTouchDown(surface, window.frameOffset() + QPoint(10, 10), touchId);
+    // Note: wl_touch.frame should not be the last event in a test until QTBUG-66563 is fixed.
+    // See also: QTBUG-66537
     compositor->sendTouchFrame(surface);
     QTRY_COMPARE(window.touchEventCount, 1);
 
     compositor->sendTouchUp(surface, touchId);
     compositor->sendTouchFrame(surface);
     QTRY_COMPARE(window.touchEventCount, 2);
+
+    QPoint mousePressPos(16, 16);
+    QCOMPARE(window.mousePressEventCount, 0);
+    compositor->sendMousePress(surface, window.frameOffset() + mousePressPos);
+    QTRY_COMPARE(window.mousePressEventCount, 1);
+    QTRY_COMPARE(window.mousePressPos, mousePressPos);
+
+    QCOMPARE(window.mouseReleaseEventCount, 0);
+    compositor->sendMouseRelease(surface);
+    QTRY_COMPARE(window.mouseReleaseEventCount, 1);
 }
 
 void tst_WaylandClient::backingStore()
@@ -266,6 +297,7 @@ public:
         m_dragIcon = QPixmap::fromImage(cursorImage);
     }
     ~DndWindow(){}
+    QPoint frameOffset() const { return QPoint(frameMargins().left(), frameMargins().top()); }
     bool dragStarted;
 
 protected:
@@ -299,14 +331,14 @@ void tst_WaylandClient::touchDrag()
     QTRY_COMPARE(QGuiApplication::focusWindow(), &window);
 
     const int id = 0;
-    compositor->sendTouchDown(surface, QPoint(10, 10), id);
+    compositor->sendTouchDown(surface, window.frameOffset() + QPoint(10, 10), id);
     compositor->sendTouchFrame(surface);
-    compositor->sendTouchMotion(surface, QPoint(20, 20), id);
+    compositor->sendTouchMotion(surface, window.frameOffset() + QPoint(20, 20), id);
     compositor->sendTouchFrame(surface);
     compositor->waitForStartDrag();
     compositor->sendDataDeviceDataOffer(surface);
-    compositor->sendDataDeviceEnter(surface, QPoint(20, 20));
-    compositor->sendDataDeviceMotion( QPoint(21, 21));
+    compositor->sendDataDeviceEnter(surface, window.frameOffset() + QPoint(20, 20));
+    compositor->sendDataDeviceMotion(window.frameOffset() + QPoint(21, 21));
     compositor->sendDataDeviceDrop(surface);
     compositor->sendDataDeviceLeave(surface);
     QTRY_VERIFY(window.dragStarted);
@@ -323,30 +355,115 @@ void tst_WaylandClient::mouseDrag()
     compositor->setKeyboardFocus(surface);
     QTRY_COMPARE(QGuiApplication::focusWindow(), &window);
 
-    compositor->sendMousePress(surface, QPoint(10, 10));
+    compositor->sendMousePress(surface, window.frameOffset() + QPoint(10, 10));
     compositor->sendDataDeviceDataOffer(surface);
-    compositor->sendDataDeviceEnter(surface, QPoint(20, 20));
-    compositor->sendDataDeviceMotion( QPoint(21, 21));
+    compositor->sendDataDeviceEnter(surface, window.frameOffset() + QPoint(20, 20));
+    compositor->sendDataDeviceMotion(window.frameOffset() + QPoint(21, 21));
     compositor->waitForStartDrag();
     compositor->sendDataDeviceDrop(surface);
     compositor->sendDataDeviceLeave(surface);
     QTRY_VERIFY(window.dragStarted);
 }
 
+void tst_WaylandClient::dontCrashOnMultipleCommits()
+{
+    auto window = new TestWindow();
+    window->show();
+
+    QRect rect(QPoint(), window->size());
+
+    QBackingStore backingStore(window);
+    backingStore.resize(rect.size());
+    backingStore.beginPaint(rect);
+    QPainter p(backingStore.paintDevice());
+    p.fillRect(rect, Qt::magenta);
+    p.end();
+    backingStore.endPaint();
+
+    backingStore.flush(rect);
+    backingStore.flush(rect);
+    backingStore.flush(rect);
+
+    compositor->processWaylandEvents();
+
+    delete window;
+
+    QTRY_VERIFY(!compositor->surface());
+}
+
+void tst_WaylandClient::hiddenTransientParent()
+{
+    QWindow parent;
+    QWindow transient;
+
+    transient.setTransientParent(&parent);
+
+    parent.show();
+    QTRY_VERIFY(compositor->surface());
+
+    parent.hide();
+    QTRY_VERIFY(!compositor->surface());
+
+    transient.show();
+    QTRY_VERIFY(compositor->surface());
+}
+
+void tst_WaylandClient::hiddenPopupParent()
+{
+    TestWindow toplevel;
+    toplevel.show();
+
+    // wl_shell relies on a mouse event in order to send a serial and seat
+    // with the set_popup request.
+    QSharedPointer<MockSurface> surface;
+    QTRY_VERIFY(surface = compositor->surface());
+    QPoint mousePressPos(16, 16);
+    QCOMPARE(toplevel.mousePressEventCount, 0);
+    compositor->sendMousePress(surface, toplevel.frameOffset() + mousePressPos);
+    QTRY_COMPARE(toplevel.mousePressEventCount, 1);
+
+    QWindow popup;
+    popup.setTransientParent(&toplevel);
+    popup.setFlag(Qt::Popup, true);
+
+    toplevel.hide();
+    QTRY_VERIFY(!compositor->surface());
+
+    popup.show();
+    QTRY_VERIFY(compositor->surface());
+}
+
+void tst_WaylandClient::glWindow()
+{
+    QSKIP("Skipping GL tests, as not supported by all CI systems: See https://bugreports.qt.io/browse/QTBUG-65802");
+
+    QScopedPointer<TestGlWindow> testWindow(new TestGlWindow);
+    testWindow->show();
+    QSharedPointer<MockSurface> surface;
+    QTRY_VERIFY(surface = compositor->surface());
+
+    //confirm we don't crash when we delete an already hidden GL window
+    //QTBUG-65553
+    testWindow->setVisible(false);
+    QTRY_VERIFY(!compositor->surface());
+}
+
 int main(int argc, char **argv)
 {
     setenv("XDG_RUNTIME_DIR", ".", 1);
     setenv("QT_QPA_PLATFORM", "wayland", 1); // force QGuiApplication to use wayland plugin
 
-    // wayland-egl hangs in the test setup when we try to initialize. Until it gets
-    // figured out, avoid clientBufferIntegration() from being called in
-    // QWaylandWindow::createDecorations().
-    setenv("QT_WAYLAND_DISABLE_WINDOWDECORATION", "1", 1);
-
     MockCompositor compositor;
     compositor.setOutputGeometry(QRect(QPoint(), screenSize));
 
     QGuiApplication app(argc, argv);
+
+    // Initializing some client buffer integrations (i.e. eglInitialize) may block while waiting
+    // for a wayland sync. So we call clientBufferIntegration prior to applicationInitialized
+    // (while the compositor processes events without waiting) in order to avoid hanging later.
+    auto *waylandIntegration = static_cast<QtWaylandClient::QWaylandIntegration *>(QGuiApplicationPrivate::platformIntegration());
+    waylandIntegration->clientBufferIntegration();
+
     compositor.applicationInitialized();
 
     tst_WaylandClient tc(&compositor);
