@@ -27,18 +27,24 @@
 ****************************************************************************/
 
 #include "mockclient.h"
+#include "mockseat.h"
+#include "mockpointer.h"
 #include "testcompositor.h"
 #include "testkeyboardgrabber.h"
-#include "testinputdevice.h"
+#include "testseat.h"
 
 #include "qwaylandview.h"
 #include "qwaylandbufferref.h"
-#include "qwaylandinput.h"
+#include "qwaylandseat.h"
 
-#include <QtWaylandCompositor/QWaylandXdgShell>
+#include <QtGui/QScreen>
+#include <QtWaylandCompositor/QWaylandXdgShellV5>
+#include <QtWaylandCompositor/QWaylandIviApplication>
+#include <QtWaylandCompositor/QWaylandIviSurface>
 #include <QtWaylandCompositor/QWaylandSurface>
 #include <QtWaylandCompositor/QWaylandResource>
 #include <qwayland-xdg-shell.h>
+#include <qwayland-ivi-application.h>
 
 #include <QtTest/QtTest>
 
@@ -48,20 +54,30 @@ class tst_WaylandCompositor : public QObject
 
 private slots:
     void init();
-    void inputDeviceCapabilities();
+    void seatCapabilities();
     void keyboardGrab();
-    void inputDeviceCreation();
-    void inputDeviceKeyboardFocus();
+    void seatCreation();
+    void seatKeyboardFocus();
+    void seatMouseFocus();
     void singleClient();
     void multipleClients();
     void geometry();
+    void modes();
+    void sizeFollowsWindow();
     void mapSurface();
     void frameCallback();
+
     void advertisesXdgShellSupport();
     void createsXdgSurfaces();
     void reportsXdgSurfaceWindowGeometry();
     void setsXdgAppId();
     void sendsXdgConfigure();
+
+    void advertisesIviApplicationSupport();
+    void createsIviSurfaces();
+    void emitsErrorOnSameIviId();
+    void sendsIviConfigure();
+    void destroysIviSurfaces();
 };
 
 void tst_WaylandCompositor::init() {
@@ -152,38 +168,38 @@ void tst_WaylandCompositor::keyboardGrab()
 
     // Set the focused surface so that key event will flow through
     QWaylandSurface *waylandSurface = compositor.surfaces.at(0);
-    QWaylandInputDevice* inputDevice = compositor.defaultInputDevice();
+    QWaylandSeat* seat = compositor.defaultSeat();
 
-    TestKeyboardGrabber* grab = static_cast<TestKeyboardGrabber *>(inputDevice->keyboard());
-    QTRY_COMPARE(grab, inputDevice->keyboard());
+    TestKeyboardGrabber* grab = static_cast<TestKeyboardGrabber *>(seat->keyboard());
+    QTRY_COMPARE(grab, seat->keyboard());
     QSignalSpy grabFocusSpy(grab, SIGNAL(focusedCalled()));
     QSignalSpy grabKeyPressSpy(grab, SIGNAL(keyPressCalled()));
     QSignalSpy grabKeyReleaseSpy(grab, SIGNAL(keyReleaseCalled()));
     //QSignalSpy grabModifierSpy(grab, SIGNAL(modifiersCalled()));
 
-    inputDevice->setKeyboardFocus(waylandSurface);
+    seat->setKeyboardFocus(waylandSurface);
     QTRY_COMPARE(grabFocusSpy.count(), 1);
 
     QKeyEvent ke(QEvent::KeyPress, Qt::Key_A, Qt::NoModifier, 30, 0, 0);
     QKeyEvent ke1(QEvent::KeyRelease, Qt::Key_A, Qt::NoModifier, 30, 0, 0);
-    inputDevice->sendFullKeyEvent(&ke);
-    inputDevice->sendFullKeyEvent(&ke1);
+    seat->sendFullKeyEvent(&ke);
+    seat->sendFullKeyEvent(&ke1);
     QTRY_COMPARE(grabKeyPressSpy.count(), 1);
     QTRY_COMPARE(grabKeyReleaseSpy.count(), 1);
 
     QKeyEvent ke2(QEvent::KeyPress, Qt::Key_Shift, Qt::NoModifier, 50, 0, 0);
     QKeyEvent ke3(QEvent::KeyRelease, Qt::Key_Shift, Qt::NoModifier, 50, 0, 0);
-    inputDevice->sendFullKeyEvent(&ke2);
-    inputDevice->sendFullKeyEvent(&ke3);
+    seat->sendFullKeyEvent(&ke2);
+    seat->sendFullKeyEvent(&ke3);
     //QTRY_COMPARE(grabModifierSpy.count(), 2);
     // Modifiers are also keys
     QTRY_COMPARE(grabKeyPressSpy.count(), 2);
     QTRY_COMPARE(grabKeyReleaseSpy.count(), 2);
 
     // Stop grabbing
-    inputDevice->setKeyboardFocus(Q_NULLPTR);
-    inputDevice->sendFullKeyEvent(&ke);
-    inputDevice->sendFullKeyEvent(&ke1);
+    seat->setKeyboardFocus(Q_NULLPTR);
+    seat->sendFullKeyEvent(&ke);
+    seat->sendFullKeyEvent(&ke1);
     QTRY_COMPARE(grabKeyPressSpy.count(), 2);
 }
 
@@ -192,12 +208,61 @@ void tst_WaylandCompositor::geometry()
     TestCompositor compositor;
     compositor.create();
 
-    QRect geometry(0, 0, 4096, 3072);
-    compositor.defaultOutput()->setGeometry(geometry);
+    QWaylandOutputMode mode(QSize(4096, 3072), 60000);
+    compositor.defaultOutput()->setPosition(QPoint(1024, 0));
+    compositor.defaultOutput()->addMode(mode, true);
+    compositor.defaultOutput()->setCurrentMode(mode);
 
     MockClient client;
 
-    QTRY_COMPARE(client.geometry, geometry);
+    QTRY_COMPARE(client.geometry, QRect(QPoint(1024, 0), QSize(4096, 3072)));
+    QTRY_COMPARE(client.resolution, QSize(4096, 3072));
+    QTRY_COMPARE(client.refreshRate, 60000);
+}
+
+void tst_WaylandCompositor::modes()
+{
+    TestCompositor compositor;
+    compositor.create();
+
+    // mode3 is current, mode4 is preferred
+    QWaylandOutputMode mode1(QSize(800, 600), 120000);
+    QWaylandOutputMode mode2(QSize(1024, 768), 100000);
+    QWaylandOutputMode mode3(QSize(1920, 1080), 60000);
+    QWaylandOutputMode mode4(QSize(2560, 1440), 59000);
+    compositor.defaultOutput()->addMode(mode1);
+    compositor.defaultOutput()->addMode(mode2, true);
+    compositor.defaultOutput()->addMode(mode3);
+    compositor.defaultOutput()->addMode(mode4, true);
+    compositor.defaultOutput()->setCurrentMode(mode3);
+
+    MockClient client;
+
+    QTRY_COMPARE(client.modes.size(), 4);
+    QTRY_COMPARE(client.currentMode, mode3);
+    QTRY_COMPARE(client.preferredMode, mode4);
+    QTRY_COMPARE(client.geometry, QRect(QPoint(0, 0), QSize(1920, 1080)));
+}
+
+void tst_WaylandCompositor::sizeFollowsWindow()
+{
+    TestCompositor compositor;
+
+    QWindow window;
+    window.resize(800, 600);
+
+    auto output = new QWaylandOutput(&compositor, &window);
+    output->setSizeFollowsWindow(true);
+
+    compositor.create();
+
+    QWaylandOutputMode mode(window.size(), qFloor(window.screen()->refreshRate() * 1000));
+
+    MockClient client;
+
+    QTRY_COMPARE(client.modes.size(), 1);
+    QTRY_COMPARE(client.currentMode, mode);
+    QTRY_COMPARE(client.preferredMode, mode);
 }
 
 void tst_WaylandCompositor::mapSurface()
@@ -212,10 +277,10 @@ void tst_WaylandCompositor::mapSurface()
 
     QWaylandSurface *waylandSurface = compositor.surfaces.at(0);
 
-    QSignalSpy mappedSpy(waylandSurface, SIGNAL(mappedChanged()));
+    QSignalSpy hasContentSpy(waylandSurface, SIGNAL(hasContentChanged()));
 
     QCOMPARE(waylandSurface->size(), QSize());
-    QCOMPARE(waylandSurface->isMapped(), false);
+    QCOMPARE(waylandSurface->hasContent(), false);
 
     QSize size(256, 256);
     ShmBuffer buffer(size, client.shm);
@@ -227,8 +292,8 @@ void tst_WaylandCompositor::mapSurface()
     wl_surface_commit(surface);
 
     QTRY_COMPARE(waylandSurface->size(), size);
-    QTRY_COMPARE(waylandSurface->isMapped(), true);
-    QTRY_COMPARE(mappedSpy.count(), 1);
+    QTRY_COMPARE(waylandSurface->hasContent(), true);
+    QTRY_COMPARE(hasContentSpy.count(), 1);
 
     wl_surface_destroy(surface);
 }
@@ -253,7 +318,7 @@ void tst_WaylandCompositor::frameCallback()
     class BufferView : public QWaylandView
     {
     public:
-        void attach(const QWaylandBufferRef &ref, const QRegion &damage) Q_DECL_OVERRIDE
+        void bufferCommitted(const QWaylandBufferRef &ref, const QRegion &damage) override
         {
             Q_UNUSED(damage);
             bufferRef = ref;
@@ -261,7 +326,7 @@ void tst_WaylandCompositor::frameCallback()
 
         QImage image() const
         {
-            if (bufferRef.isNull() || !bufferRef.isShm())
+            if (bufferRef.isNull() || !bufferRef.isSharedMemory())
                 return QImage();
             return bufferRef.image();
         }
@@ -296,7 +361,7 @@ void tst_WaylandCompositor::frameCallback()
         wl_surface_damage(surface, 0, 0, size.width(), size.height());
         wl_surface_commit(surface);
 
-        QTRY_COMPARE(waylandSurface->isMapped(), true);
+        QTRY_COMPARE(waylandSurface->hasContent(), true);
         QTRY_COMPARE(damagedSpy.count(), i + 1);
 
         QCOMPARE(static_cast<BufferView*>(waylandSurface->views().first())->image(), buffer.image);
@@ -309,7 +374,7 @@ void tst_WaylandCompositor::frameCallback()
     wl_surface_destroy(surface);
 }
 
-void tst_WaylandCompositor::inputDeviceCapabilities()
+void tst_WaylandCompositor::seatCapabilities()
 {
     TestCompositor compositor;
     compositor.create();
@@ -317,19 +382,19 @@ void tst_WaylandCompositor::inputDeviceCapabilities()
     MockClient client;
     Q_UNUSED(client);
 
-    QWaylandInputDevice dev(&compositor, QWaylandInputDevice::Pointer);
+    QWaylandSeat dev(&compositor, QWaylandSeat::Pointer);
 
     QTRY_VERIFY(dev.pointer());
     QTRY_VERIFY(!dev.keyboard());
     QTRY_VERIFY(!dev.touch());
 
-    QWaylandInputDevice dev2(&compositor, QWaylandInputDevice::Keyboard | QWaylandInputDevice::Touch);
+    QWaylandSeat dev2(&compositor, QWaylandSeat::Keyboard | QWaylandSeat::Touch);
     QTRY_VERIFY(!dev2.pointer());
     QTRY_VERIFY(dev2.keyboard());
     QTRY_VERIFY(dev2.touch());
 }
 
-void tst_WaylandCompositor::inputDeviceCreation()
+void tst_WaylandCompositor::seatCreation()
 {
     TestCompositor compositor(true);
     compositor.create();
@@ -337,23 +402,24 @@ void tst_WaylandCompositor::inputDeviceCreation()
     MockClient client;
     Q_UNUSED(client);
 
-    TestInputDevice* dev = static_cast<TestInputDevice*>(compositor.defaultInputDevice());
+    TestSeat* seat = qobject_cast<TestSeat *>(compositor.defaultSeat());
+    QTRY_VERIFY(seat);
 
     // The compositor will create the default input device
-    QTRY_COMPARE(compositor.defaultInputDevice(), dev);
+    QTRY_VERIFY(seat->isInitialized());
 
     QList<QMouseEvent *> allEvents;
-    allEvents += dev->createMouseEvents(5);
+    allEvents += seat->createMouseEvents(5);
     foreach (QMouseEvent *me, allEvents) {
-        compositor.inputDeviceFor(me);
+        compositor.seatFor(me);
     }
 
     // The default input device will get called exatly the number of times it has created
     // the events
-    QTRY_COMPARE(dev->queryCount(), 5);
+    QTRY_COMPARE(seat->queryCount(), 5);
 }
 
-void tst_WaylandCompositor::inputDeviceKeyboardFocus()
+void tst_WaylandCompositor::seatKeyboardFocus()
 {
     TestCompositor compositor(true);
     compositor.create();
@@ -365,21 +431,74 @@ void tst_WaylandCompositor::inputDeviceKeyboardFocus()
     QTRY_COMPARE(compositor.surfaces.size(), 1);
 
     QWaylandSurface *waylandSurface = compositor.surfaces.at(0);
-    QWaylandInputDevice* dev = compositor.defaultInputDevice();
+    QWaylandSeat* dev = compositor.defaultSeat();
     dev->setKeyboardFocus(waylandSurface);
-    QTRY_COMPARE(compositor.defaultInputDevice()->keyboardFocus(), waylandSurface);
+    QTRY_COMPARE(compositor.defaultSeat()->keyboardFocus(), waylandSurface);
 
     wl_surface_destroy(surface);
     QTRY_VERIFY(compositor.surfaces.size() == 0);
 
-    QTRY_VERIFY(!compositor.defaultInputDevice()->keyboardFocus());
+    QTRY_VERIFY(!compositor.defaultSeat()->keyboardFocus());
+}
+
+void tst_WaylandCompositor::seatMouseFocus()
+{
+    TestCompositor compositor(true);
+    compositor.create();
+
+    // Create client after all the seats have been set up as the mock client
+    // does not dynamically listen to new seats
+    MockClient client;
+    wl_surface *surface = client.createSurface();
+    QTRY_COMPARE(compositor.surfaces.size(), 1);
+
+    QWaylandSurface *waylandSurface = compositor.surfaces.at(0);
+    auto view = new QWaylandView;
+    view->setSurface(waylandSurface);
+
+    QWaylandSeat* seat = compositor.defaultSeat();
+    seat->setMouseFocus(view);
+    seat->sendMouseMoveEvent(view, QPointF(10, 10), QPointF(100, 100));
+
+    compositor.flushClients();
+
+    QTRY_VERIFY(seat->mouseFocus());
+    QTRY_VERIFY(seat->pointer());
+    QTRY_COMPARE(seat->mouseFocus()->surface(), waylandSurface);
+
+    QTRY_COMPARE(client.m_seats.size(), 1);
+    MockPointer *mockPointer = client.m_seats.first()->pointer();
+    QVERIFY(mockPointer);
+    QTRY_COMPARE(mockPointer->m_enteredSurface, surface);
+
+    delete view;
+
+    compositor.flushClients();
+
+    QTRY_COMPARE(mockPointer->m_enteredSurface, nullptr);
+    QTRY_VERIFY(!compositor.defaultSeat()->mouseFocus());
+
+    view = new QWaylandView;
+    view->setSurface(waylandSurface);
+    seat->sendMouseMoveEvent(view, QPointF(10, 10), QPointF(100, 100));
+    QTRY_COMPARE(compositor.defaultSeat()->mouseFocus(), view);
+
+    compositor.flushClients();
+
+    QTRY_COMPARE(mockPointer->m_enteredSurface, surface);
+
+    wl_surface_destroy(surface);
+    QTRY_VERIFY(compositor.surfaces.size() == 0);
+    QTRY_VERIFY(!compositor.defaultSeat()->mouseFocus());
+
+    delete view;
 }
 
 class XdgTestCompositor: public TestCompositor {
     Q_OBJECT
 public:
     XdgTestCompositor() : xdgShell(this) {}
-    QWaylandXdgShell xdgShell;
+    QWaylandXdgShellV5 xdgShell;
 };
 
 void tst_WaylandCompositor::advertisesXdgShellSupport()
@@ -388,7 +507,7 @@ void tst_WaylandCompositor::advertisesXdgShellSupport()
     compositor.create();
 
     MockClient client;
-    QTRY_VERIFY(&client.xdgShell);
+    QTRY_VERIFY(client.xdgShell);
 }
 
 void tst_WaylandCompositor::createsXdgSurfaces()
@@ -397,11 +516,11 @@ void tst_WaylandCompositor::createsXdgSurfaces()
     compositor.create();
 
     MockClient client;
-    QTRY_VERIFY(&client.xdgShell);
+    QTRY_VERIFY(client.xdgShell);
 
-    QSignalSpy xdgSurfaceCreatedSpy(&compositor.xdgShell, &QWaylandXdgShell::xdgSurfaceCreated);
-    QWaylandXdgSurface *xdgSurface = nullptr;
-    QObject::connect(&compositor.xdgShell, &QWaylandXdgShell::xdgSurfaceCreated, [&](QWaylandXdgSurface *s) {
+    QSignalSpy xdgSurfaceCreatedSpy(&compositor.xdgShell, &QWaylandXdgShellV5::xdgSurfaceCreated);
+    QWaylandXdgSurfaceV5 *xdgSurface = nullptr;
+    QObject::connect(&compositor.xdgShell, &QWaylandXdgShellV5::xdgSurfaceCreated, [&](QWaylandXdgSurfaceV5 *s) {
         xdgSurface = s;
     });
 
@@ -417,8 +536,8 @@ void tst_WaylandCompositor::reportsXdgSurfaceWindowGeometry()
     XdgTestCompositor compositor;
     compositor.create();
 
-    QWaylandXdgSurface *xdgSurface = nullptr;
-    QObject::connect(&compositor.xdgShell, &QWaylandXdgShell::xdgSurfaceCreated, [&](QWaylandXdgSurface *s) {
+    QWaylandXdgSurfaceV5 *xdgSurface = nullptr;
+    QObject::connect(&compositor.xdgShell, &QWaylandXdgShellV5::xdgSurfaceCreated, [&](QWaylandXdgSurfaceV5 *s) {
         xdgSurface = s;
     });
 
@@ -445,8 +564,8 @@ void tst_WaylandCompositor::setsXdgAppId()
     XdgTestCompositor compositor;
     compositor.create();
 
-    QWaylandXdgSurface *xdgSurface = nullptr;
-    QObject::connect(&compositor.xdgShell, &QWaylandXdgShell::xdgSurfaceCreated, [&](QWaylandXdgSurface *s) {
+    QWaylandXdgSurfaceV5 *xdgSurface = nullptr;
+    QObject::connect(&compositor.xdgShell, &QWaylandXdgShellV5::xdgSurfaceCreated, [&](QWaylandXdgSurfaceV5 *s) {
         xdgSurface = s;
     });
 
@@ -466,7 +585,7 @@ void tst_WaylandCompositor::sendsXdgConfigure()
     {
     public:
         MockXdgSurface(::xdg_surface *xdgSurface) : QtWayland::xdg_surface(xdgSurface) {}
-        void xdg_surface_configure(int32_t width, int32_t height, wl_array *rawStates, uint32_t serial) Q_DECL_OVERRIDE
+        void xdg_surface_configure(int32_t width, int32_t height, wl_array *rawStates, uint32_t serial) override
         {
             configureSize = QSize(width, height);
             configureSerial = serial;
@@ -487,8 +606,8 @@ void tst_WaylandCompositor::sendsXdgConfigure()
     XdgTestCompositor compositor;
     compositor.create();
 
-    QWaylandXdgSurface *xdgSurface = nullptr;
-    QObject::connect(&compositor.xdgShell, &QWaylandXdgShell::xdgSurfaceCreated, [&](QWaylandXdgSurface *s) {
+    QWaylandXdgSurfaceV5 *xdgSurface = nullptr;
+    QObject::connect(&compositor.xdgShell, &QWaylandXdgShellV5::xdgSurfaceCreated, [&](QWaylandXdgSurfaceV5 *s) {
         xdgSurface = s;
     });
 
@@ -503,15 +622,15 @@ void tst_WaylandCompositor::sendsXdgConfigure()
     QTRY_VERIFY(!xdgSurface->fullscreen());
     QTRY_VERIFY(!xdgSurface->resizing());
 
-    xdgSurface->sendConfigure(QSize(10, 20), QVector<QWaylandXdgSurface::State>{QWaylandXdgSurface::State::ActivatedState});
+    xdgSurface->sendConfigure(QSize(10, 20), QVector<QWaylandXdgSurfaceV5::State>{QWaylandXdgSurfaceV5::State::ActivatedState});
     compositor.flushClients();
-    QTRY_COMPARE(mockXdgSurface.configureStates, QList<uint>{QWaylandXdgSurface::State::ActivatedState});
+    QTRY_COMPARE(mockXdgSurface.configureStates, QList<uint>{QWaylandXdgSurfaceV5::State::ActivatedState});
     QTRY_COMPARE(mockXdgSurface.configureSize, QSize(10, 20));
 
-    xdgSurface->requestMaximized(QSize(800, 600));
+    xdgSurface->sendMaximized(QSize(800, 600));
     compositor.flushClients();
-    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::MaximizedState));
-    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::ActivatedState));
+    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::MaximizedState));
+    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::ActivatedState));
     QTRY_COMPARE(mockXdgSurface.configureSize, QSize(800, 600));
 
     // There hasn't been any ack_configures, so state should still be unchanged
@@ -523,44 +642,44 @@ void tst_WaylandCompositor::sendsXdgConfigure()
     QTRY_VERIFY(xdgSurface->activated());
     QTRY_VERIFY(xdgSurface->maximized());
 
-    xdgSurface->requestUnMaximized();
+    xdgSurface->sendUnmaximized();
     compositor.flushClients();
-    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::MaximizedState));
-    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::ActivatedState));
+    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::MaximizedState));
+    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::ActivatedState));
     QTRY_COMPARE(mockXdgSurface.configureSize, QSize(0, 0));
 
     // The unmaximized configure hasn't been acked, so maximized should still be true
     QTRY_VERIFY(xdgSurface->maximized());
     QTRY_VERIFY(xdgSurface->activated());
 
-    xdgSurface->requestResizing(QSize(800, 600));
+    xdgSurface->sendResizing(QSize(800, 600));
     compositor.flushClients();
-    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::ResizingState));
+    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::ResizingState));
     QTRY_COMPARE(mockXdgSurface.configureSize, QSize(800, 600));
 
-    xdgSurface->requestFullscreen(QSize(1024, 768));
+    xdgSurface->sendFullscreen(QSize(1024, 768));
     compositor.flushClients();
-    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::ActivatedState));
-    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::FullscreenState));
+    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::ActivatedState));
+    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::FullscreenState));
     QTRY_COMPARE(mockXdgSurface.configureSize, QSize(1024, 768));
     uint fullscreenSerial = mockXdgSurface.configureSerial;
 
-    xdgSurface->requestUnMaximized();
+    xdgSurface->sendUnmaximized();
     compositor.flushClients();
-    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::ActivatedState));
-    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::FullscreenState));
+    QTRY_VERIFY(mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::ActivatedState));
+    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::FullscreenState));
 
-    xdgSurface->sendConfigure(QSize(0, 0), QVector<QWaylandXdgSurface::State>{});
+    xdgSurface->sendConfigure(QSize(0, 0), QVector<QWaylandXdgSurfaceV5::State>{});
     compositor.flushClients();
-    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::ActivatedState));
+    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::ActivatedState));
 
-    xdgSurface->requestMaximized(QSize(800, 600));
+    xdgSurface->sendMaximized(QSize(800, 600));
     compositor.flushClients();
-    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::ActivatedState));
+    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::ActivatedState));
 
-    xdgSurface->requestFullscreen(QSize(800, 600));
+    xdgSurface->sendFullscreen(QSize(800, 600));
     compositor.flushClients();
-    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurface::State::MaximizedState));
+    QTRY_VERIFY(!mockXdgSurface.configureStates.contains(QWaylandXdgSurfaceV5::State::MaximizedState));
 
     // Verify that acking a configure that's not the most recently sent works
     xdg_surface_ack_configure(clientXdgSurface, fullscreenSerial);
@@ -572,5 +691,149 @@ void tst_WaylandCompositor::sendsXdgConfigure()
     QTRY_VERIFY(!xdgSurface->resizing());
 }
 
+class IviTestCompositor: public TestCompositor {
+    Q_OBJECT
+public:
+    IviTestCompositor() : iviApplication(this) {}
+    QWaylandIviApplication iviApplication;
+};
+
+void tst_WaylandCompositor::advertisesIviApplicationSupport()
+{
+    IviTestCompositor compositor;
+    compositor.create();
+
+    MockClient client;
+    QTRY_VERIFY(client.iviApplication);
+}
+
+void tst_WaylandCompositor::createsIviSurfaces()
+{
+    IviTestCompositor compositor;
+    compositor.create();
+
+    MockClient client;
+    QTRY_VERIFY(client.iviApplication);
+
+    QSignalSpy iviSurfaceCreatedSpy(&compositor.iviApplication, &QWaylandIviApplication::iviSurfaceRequested);
+    QWaylandIviSurface *iviSurface = nullptr;
+    QObject::connect(&compositor.iviApplication, &QWaylandIviApplication::iviSurfaceCreated, [&](QWaylandIviSurface *s) {
+        iviSurface = s;
+    });
+
+    wl_surface *surface = client.createSurface();
+    client.createIviSurface(surface, 123);
+    QTRY_COMPARE(iviSurfaceCreatedSpy.count(), 1);
+    QTRY_VERIFY(iviSurface);
+    QTRY_VERIFY(iviSurface->surface());
+    QTRY_COMPARE(iviSurface->iviId(), 123u);
+}
+
+void tst_WaylandCompositor::emitsErrorOnSameIviId()
+{
+    IviTestCompositor compositor;
+    compositor.create();
+
+    {
+        MockClient firstClient;
+        QTRY_VERIFY(&firstClient.iviApplication);
+
+        QWaylandIviSurface *firstIviSurface = nullptr;
+        QObject::connect(&compositor.iviApplication, &QWaylandIviApplication::iviSurfaceCreated, [&](QWaylandIviSurface *s) {
+            firstIviSurface = s;
+        });
+
+        firstClient.createIviSurface(firstClient.createSurface(), 123);
+        QTRY_VERIFY(firstIviSurface);
+        QTRY_COMPARE(firstIviSurface->iviId(), 123u);
+
+        {
+            MockClient secondClient;
+            QTRY_VERIFY(&secondClient.iviApplication);
+            QTRY_COMPARE(compositor.clients().count(), 2);
+
+            secondClient.createIviSurface(secondClient.createSurface(), 123);
+            compositor.flushClients();
+
+            QTRY_COMPARE(secondClient.error, EPROTO);
+            QTRY_COMPARE(secondClient.protocolError.interface, &ivi_application_interface);
+            QTRY_COMPARE(static_cast<ivi_application_error>(secondClient.protocolError.code), IVI_APPLICATION_ERROR_IVI_ID);
+            QTRY_COMPARE(compositor.clients().count(), 1);
+        }
+    }
+
+    // The other clients have passed out of scope and have been destroyed,
+    // it should now be ok to create new application with the same id
+    MockClient thirdClient;
+    QTRY_VERIFY(&thirdClient.iviApplication);
+
+    QWaylandIviSurface *thirdIviSurface = nullptr;
+    QObject::connect(&compositor.iviApplication, &QWaylandIviApplication::iviSurfaceCreated, [&](QWaylandIviSurface *s) {
+        thirdIviSurface = s;
+    });
+    thirdClient.createIviSurface(thirdClient.createSurface(), 123);
+    compositor.flushClients();
+
+    QTRY_VERIFY(thirdIviSurface);
+    QTRY_COMPARE(thirdIviSurface->iviId(), 123u);
+    QTRY_COMPARE(thirdClient.error, 0);
+}
+
+void tst_WaylandCompositor::sendsIviConfigure()
+{
+    class MockIviSurface : public QtWayland::ivi_surface
+    {
+    public:
+        MockIviSurface(::ivi_surface *iviSurface) : QtWayland::ivi_surface(iviSurface) {}
+        void ivi_surface_configure(int32_t width, int32_t height) override
+        {
+            configureSize = QSize(width, height);
+        }
+        QSize configureSize;
+    };
+
+    IviTestCompositor compositor;
+    compositor.create();
+
+    MockClient client;
+    QTRY_VERIFY(client.iviApplication);
+
+    QWaylandIviSurface *iviSurface = nullptr;
+    QObject::connect(&compositor.iviApplication, &QWaylandIviApplication::iviSurfaceCreated, [&](QWaylandIviSurface *s) {
+        iviSurface = s;
+    });
+
+    wl_surface *surface = client.createSurface();
+    ivi_surface *clientIviSurface = client.createIviSurface(surface, 123);
+    MockIviSurface mockIviSurface(clientIviSurface);
+
+    QTRY_VERIFY(iviSurface);
+    iviSurface->sendConfigure(QSize(800, 600));
+    compositor.flushClients();
+
+    QTRY_COMPARE(mockIviSurface.configureSize, QSize(800, 600));
+}
+
+void tst_WaylandCompositor::destroysIviSurfaces()
+{
+    IviTestCompositor compositor;
+    compositor.create();
+
+    MockClient client;
+    QTRY_VERIFY(client.iviApplication);
+
+    QWaylandIviSurface *iviSurface = nullptr;
+    QObject::connect(&compositor.iviApplication, &QWaylandIviApplication::iviSurfaceCreated, [&](QWaylandIviSurface *s) {
+        iviSurface = s;
+    });
+
+    QtWayland::ivi_surface mockIviSurface(client.createIviSurface(client.createSurface(), 123));
+    QTRY_VERIFY(iviSurface);
+
+    QSignalSpy destroySpy(iviSurface, SIGNAL(destroyed()));
+    mockIviSurface.destroy();
+    QTRY_VERIFY(destroySpy.count() == 1);
+}
+
 #include <tst_compositor.moc>
 QTEST_MAIN(tst_WaylandCompositor);
